Package: priceR
Type: Package
Title: Economics and Pricing Tools
Version: 1.0.4
Imports: dplyr, gsubfn, stringr, purrr, jsonlite, stats, lubridate,
        stringi, tidyr
Authors@R: c(person(given="Steve", family="Condylios",
    role=c("aut", "cre"), email="steve.condylios@gmail.com",
    ),
    person(given="Bruno", family="Mioto", role="ctb", email="brunomioto97@gmail.com"),
    person(given = "Bryan", family = "Shalloway", role = "ctb", email = "brshallodev@gmail.com"))
Maintainer: Steve Condylios <steve.condylios@gmail.com>
BugReports: https://github.com/stevecondylios/priceR/issues
License: MIT + file LICENSE
URL: https://github.com/stevecondylios/priceR
Description: Functions to aid in micro and macro economic analysis and handling of price and
    currency data. Includes extraction of relevant inflation and exchange rate data from World Bank
    API, data cleaning/parsing, and standardisation. Inflation adjustment
    calculations as found in Principles of Macroeconomics by Gregory Mankiw et al (2014). Current
    and historical end of day exchange rates for 171 currencies from the European Central Bank
    Statistical Data Warehouse (2020).
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: testthat
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-11 03:33:12 UTC; st
Author: Steve Condylios [aut, cre],
  Bruno Mioto [ctb],
  Bryan Shalloway [ctb]
Repository: CRAN
Date/Publication: 2026-05-11 03:50:02 UTC
Built: R 4.6.0; ; 2026-05-11 06:54:38 UTC; unix

# Generated by roxygen2: do not edit by hand

export(adjust_for_inflation)
export(afi)
export(append_exchangeratehost_access_key)
export(convert_currencies)
export(convert_to_iso2Code)
export(country_input_type)
export(currencies)
export(currency_characters)
export(currency_to_numeric)
export(exchange_rate_latest)
export(extract_salary)
export(format_currency)
export(format_dollars)
export(from_to_dates_rates)
export(historical_exchange_rates)
export(pminmax)
export(retrieve_inflation_data)
export(round_down_to_nearest)
export(round_to_nearest)
export(round_up_to_nearest)
export(show_countries)
export(url_all_results)
import(dplyr)
import(gsubfn)
import(purrr)
import(stringi)
import(stringr)
importFrom(jsonlite,fromJSON)
importFrom(purrr,map_df)
importFrom(stats,na.omit)
importFrom(utils,stack)

# priceR <a href='https://github.com/stevecondylios/priceR'><img src='man/figures/priceR.png' align="right" height="139" /></a>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/priceR)](https://cran.r-project.org/package=priceR)
[![R build
status](https://github.com/stevecondylios/priceR/workflows/R-CMD-check/badge.svg)](https://github.com/stevecondylios/priceR/actions?workflow=R-CMD-check)
<!-- badges: end -->

`priceR` contains 4 types of capabilties:

- *Exchange Rates* - easily retrieve exchange rates for immediate use
- *Inflation* - easily inflate past (nominal) values into present day
  (real) prices
- *Regular Expressions* - easily extract common pricing patterns from
  free text
- *Formatting* - easily handle currencies in written work, including
  Rmarkdown documents

### Installation

Installation via CRAN `install.packages("priceR")`

``` r
library(priceR)
library(tidyverse)
options(scipen = 100); options(digits = 6)
```

# Exchange rates

### Setup

Set up only takes a minute and is free for 100 requests per account per
calendar month.

Go to <https://exchangerate.host/>, create a free account, and replace
`7e5e3140140bd8e4f4650cc41fc772c0` with your API key in the following,
and run once per R session.

``` r
Sys.setenv("EXCHANGERATEHOST_ACCESS_KEY"="7e5e3140140bd8e4f4650cc41fc772c0")
```

### Current exchange rates

View the current exchange rates for 170 currencies (see them all by
running `currencies()`):

``` r
exchange_rate_latest("USD") %>%
  head(10)
```

    ##    currency one_USD_is_equivalent_to
    ## 1       AED                  3.67250
    ## 2       AFN                 65.49843
    ## 3       ALL                 83.07496
    ## 4       AMD                384.26342
    ## 5       ANG                  1.78998
    ## 6       AOA                916.99990
    ## 7       ARS               1432.54310
    ## 8       AUD                  1.52554
    ## 9       AWG                  1.80250
    ## 10      AZN                  1.70556

### Historical exchange rates

Here’s an example of how to get exchange rates for some currency pairs:

``` r
# Retrieve AUD to USD exchange rates
au <- historical_exchange_rates("AUD", to = "USD",
                          start_date = "2013-01-01", end_date = "2023-06-30")

# Retrieve AUD to EUR exchange rates
ae <- historical_exchange_rates("AUD", to = "EUR",
                          start_date = "2013-01-01", end_date = "2023-06-30")

# Combine
cur <- au %>% left_join(ae, by = "date")

head(cur)
```

    ##         date one_AUD_equivalent_to_x_USD one_AUD_equivalent_to_x_EUR
    ## 1 2013-01-01                     1.03746                    0.785518
    ## 2 2013-01-02                     1.04997                    0.795034
    ## 3 2013-01-03                     1.04722                    0.800920
    ## 4 2013-01-04                     1.04837                    0.800502
    ## 5 2013-01-05                     1.04837                    0.800502
    ## 6 2013-01-06                     1.04916                    0.800944

And to plot the exchange rate data:

``` r
library(ggplot2)
library(ggthemes)
library(ggrepel)

cur %>%
  rename(aud_to_usd = one_AUD_equivalent_to_x_USD,
         aud_to_eur = one_AUD_equivalent_to_x_EUR) %>%
  pivot_longer(c("aud_to_usd", "aud_to_eur")) %>%
  mutate(date = as.Date(date)) %>%
  ggplot(aes(x=date, y = value, colour=name)) +
  geom_line(size=1) +
  scale_color_manual(
    breaks = c("aud_to_usd", "aud_to_eur"), # Sets order in legend
    labels = c( "AUD to USD", "AUD to EUR"), # Pretty names in legend
    values = c("#02506A", "#03A5DC") # Sets line/legend colours
    ) +
  scale_x_date(date_labels = "%b %Y", date_breaks = "6 month") +
  scale_y_continuous(
    expand = c(0, 0),
    limits = c(0, 1.5)
    ) +
  labs(
    title = "AUD to USD and EUR 2013 to 2023",
    subtitle = "Plotting the Australian Dollar against the USD and Euro",
    y = "Exchange Rate"
    ) +
  theme_economist() +
  theme(
    plot.title = element_text(size = 18, margin=margin(0,0,8,0)),
    axis.title.x = element_blank(),
    axis.ticks.x = element_blank(),
    axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1),
    axis.title.y = element_text(vjust = 3.5),
    legend.position="bottom",
    legend.title = element_blank()
    )
```

    ## Warning: Using `size` aesthetic for lines was deprecated in ggplot2 3.4.0.
    ## ℹ Please use `linewidth` instead.
    ## This warning is displayed once every 8 hours.
    ## Call `lifecycle::last_lifecycle_warnings()` to see where this warning was
    ## generated.

![](README_files/figure-gfm/unnamed-chunk-4-1.png)<!-- -->

``` r
cur %>%
  tail(200) %>%
  rename(aud_to_usd = one_AUD_equivalent_to_x_USD,
         aud_to_eur = one_AUD_equivalent_to_x_EUR) %>%
  mutate(date = as.Date(date)) %>%
  ggplot(aes(x = date, y = aud_to_usd, group = 1)) +
  geom_line(colour = "#F15B40") +
  geom_smooth(method = 'loess', colour="#03A5DC") +
  scale_x_date(date_labels = "%b %Y", date_breaks = "1 month") +
  labs(
    title = "AUD to USD over last 200 days",
    subtitle = "AUD to USD Exchange Rate; Polynomial regression trendline",
    y = "Exchange Rate"
    ) +
  theme_economist() +
  theme(
    plot.title = element_text(size = 18, margin=margin(0,0,8,0)),
    axis.title.x = element_blank(),
    axis.ticks.x = element_blank(),
    axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1),
    axis.title.y = element_text(vjust = 3.5),
    legend.position="bottom",
    legend.title = element_blank()
    )
```

![](README_files/figure-gfm/unnamed-chunk-5-1.png)<!-- -->

``` r
cur %>%
  tail(365 * 8) %>%
  rename(aud_to_usd = one_AUD_equivalent_to_x_USD,
         aud_to_eur = one_AUD_equivalent_to_x_EUR) %>%
  mutate(date = as.Date(date)) %>%
  ggplot(aes(x = date, y = aud_to_eur, group = 1)) +
  geom_line() +
  geom_smooth(method = 'loess', se = TRUE) +
  geom_line(colour = "#02506A") +
  geom_smooth(method = 'loess', colour="#03A5DC") +
  scale_x_date(date_labels = "%Y", date_breaks = "1 year") +
  labs(
    title = "AUD to EUR over last 8 years",
    subtitle = "AUD to EUR Exchange Rate; Polynomial regression trendline",
    y = "Exchange Rate"
    ) +
  theme_economist() +
  theme(
    plot.title = element_text(size = 18, margin=margin(0,0,8,0)),
    axis.title.x = element_blank(),
    axis.ticks.x = element_blank(),
    axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1),
    axis.title.y = element_text(vjust = 3.5),
    legend.position="bottom",
    legend.title = element_blank()
    )
```

![](README_files/figure-gfm/unnamed-chunk-6-1.png)<!-- -->

# Inflation

### Adjust prices for inflation

`adjust_for_inflation()` automatically converts between nominal and real
dollars, or in/deflates prices from one year’s prices to another’s.

It works for 304 countries / areas (see them with all by running
`show_countries()`).

``` r
set.seed(123)
nominal_prices <- rnorm(10, mean=10, sd=3)
years <- round(rnorm(10, mean=2006, sd=5))
df <- data.frame(years, nominal_prices)

df$in_2008_dollars <- adjust_for_inflation(nominal_prices, years, "US", to_date = 2008)
```

    ## Generating URL to request all 296 results
    ## Retrieving inflation data for US 
    ## Generating URL to request all 65 results

``` r
df
```

    ##    years nominal_prices in_2008_dollars
    ## 1   2012        8.31857         7.66782
    ## 2   2008        9.30947         9.30947
    ## 3   2008       14.67612        14.67612
    ## 4   2007       10.21153        10.60356
    ## 5   2003       10.38786        12.15782
    ## 6   2015       15.14519        13.26473
    ## 7   2008       11.38275        11.38275
    ## 8   1996        6.20482         8.51713
    ## 9   2010        7.93944         7.67319
    ## 10  2004        8.66301         9.87471

# Extraction helpers

These helpers let you extract useful *numeric* data from messy free text
(character) data.

### Extract salary from free text

`extract_salary()` extracts salaries as useful numeric data from
non-standard free text

``` r
messy_salary_data <- c(
  "$90000 - $120000 per annum",
  "$90k - $110k p.a.",
  "$110k - $120k p.a. + super + bonus + benefits",
  "$140K-$160K + Super + Bonus/Equity",
  "$200,000 - $250,000 package",
  "c$200K Package Neg",
  "$700 p/d",                                       # daily
  "$120 - $140 (Inc. Super) per hour",              # hourly
  "Competitive"                                     # nothing useful (will return NA)
)

messy_salary_data %>%
  extract_salary(include_periodicity = TRUE,
                 salary_range_handling = "average")
```

    ##   salary periodicity
    ## 1 105000      Annual
    ## 2 100000      Annual
    ## 3 115000      Annual
    ## 4 150000      Annual
    ## 5 225000      Annual
    ## 6 200000      Annual
    ## 7 175000       Daily
    ## 8 260000      Hourly
    ## 9     NA      Annual

# Formatting helpers

### Neatly format currencies

`format_currency()` makes nicely formats numeric data:

``` r
format_currency("22500000", "¥")
```

    ## [1] "¥22,500,000"

`format_dollars()` is the same but exclusively for dollars:

``` r
format_dollars(c("445.50", "199.99"), digits = 2)
```

    ## [1] "$445.50" "$199.99"

# More about priceR

### Research and academia

Curran-Groome, W., Hino, M., BenDor, T. and Salvesen, D., 2022.
[*Complexities and costs of floodplain buyout
implementation*](https://doi.org/10.1016/j.landusepol.2022.106128), Land
Use Policy, Volume 118, July 2022.

Thomas, C., Shae, W., Koestler, D., DeFor, T., Bahr, N. and Alpern, J.,
2022. [*Antifungal drug price increases in the United States
2000–2019*](https://pubmed.ncbi.nlm.nih.gov/35722703/), Mycoses, Online
Ahead of Print, June 2022.

Petitbon, A. and Hitchcock, D., 2022. [*What Kind of Music Do You Like?
A Statistical Analysis of Music Genre Popularity Over
Time*](https://people.stat.sc.edu/hitchcock/jds1040.pdf), Journal of
Data Science, Volume 20 (2), April 2022.

Widdicombe, J., Basáñez, M., Entezami, M., Jackson, D., Larrieu, E. and
Prada, J., 2022. [*The economic evaluation of Cystic echinococcosis
control strategies focused on zoonotic hosts: A scoping
review*](https://journals.plos.org/plosntds/article?id=10.1371/journal.pntd.0010568),
PLoS Neglected Tropical Diseases, Volume 16 (7), July 2022.

Thielen, F.W., Heine, R.J.S.D., Berg, S. van den, Ham, R.M.T. ten and
Groot, C.A.U. (2022). [Towards sustainability and affordability of
expensive cell and gene therapies? Applying a cost-based pricing model
to estimate prices for Libmeldy and
Zolgensma](https://doi.org/10.1016/j.jcyt.2022.09.002), Cytotherapy
\[online\] <doi:10.1016/j.jcyt.2022.09.002>.

### Guides and tutorials

How to [*Convert Between Currencies Using
priceR*](https://www.bryanshalloway.com/2022/06/16/converting-between-currencies-using-pricer/)
by Bryan Shalloway

### Contributing to priceR

If your research, guide or tutorial uses priceR, please contact the
priceR maintainer (or create an
[issue](https://github.com/stevecondylios/priceR/issues)) so your work
can be included here.

**Issues and Feature Requests**

When reporting an issue, please include:

- Example code that reproduces the **observed** behavior.
- An explanation of what the **expected** behavior is.

For feature requests, raise an issue with the following:

- The desired functionality
- Example inputs and desired output

**Pull Requests**

Pull requests are welcomed. Before doing so, please create an issue or
email me with your idea.

Any new functions should follow the conventions established by the the
package’s existing functions. Please ensure

- Functions are sensibly named
- The **intent** of the contribution is clear
- At least one example is provided in the documentation

#' Display link to further information
#'
#'
#' @name display_api_info
#'
#' @usage display_api_info()
#'
#'
#' @examples
#' \dontrun{
#' # Display a message indicating where further documentation can be found
#' }
#'



display_api_info <- function() {

      if(getOption("forex_warning", TRUE)) {
      # This message appears on first use only of this/similar functions
      message(paste(
              "For full currency exchange rate API documentation visit:\n",
              "https://exchangerate.host/#/#docs\n",
              "(this message will only appear once per session)"))
      options("forex_warning"=FALSE)
  }

}










#' Retrieves exchangerate.host (forex) API key from R environment variables
#' and appends to API call
#' @name append_exchangeratehost_access_key
#'
#' @usage append_exchangeratehost_access_key(url)
#'
#' @return The input URL with API access key appended as a URL parameter
#'
#' @param url A URL representing an API endpoint
#' @export
#'
#' @examples
#' \dontrun{
#' base_Url <- "https://api.exchangerate.host/latest?base=USD"
#' base_Url %>% append_exchangeratehost_access_key
#'
#' # [1] "https://api.exchangerate.host/latest?base=USD&access_key=7e5e3140140bd8e4f4650cc41fc772c0"
#'
#' }
#'
#'


append_exchangeratehost_access_key <- function(url) {

  key = Sys.getenv("EXCHANGERATEHOST_ACCESS_KEY")

  if(key == "") {
    error_message = "
    Exchangerate.host access key not found.
    Please create one here: https://exchangerate.host/
    Then set it in your R environment with
    Sys.setenv(\"EXCHANGERATEHOST_ACCESS_KEY\"=\"your_access_key\")
    E.g.
    Sys.setenv(\"EXCHANGERATEHOST_ACCESS_KEY\"=\"afty4m2dmpcs1uyp9coimr5vjr4y27t2\")
    "
    stop(error_message)
  }

  if (grepl("\\?", url)) {
    paste0(url, "&access_key=", key)
  } else {
    paste0(url, "?access_key=", key)
  }

}







#' Retrieve available currencies and their respective symbols/codes
#' @name currencies
#'
#' @usage currencies()
#'
#' @return A data.frame of available currencies and their respective symbols/codes
#'
#' @import dplyr purrr
#' @importFrom jsonlite fromJSON
#'
#' @export
#'
#' @examples
#' \dontrun{
#' # Display available currencies and their respective symbols/codes
#' currencies()
#' #                                description code
#' # 1              United Arab Emirates Dirham  AED
#' # 2                           Afghan Afghani  AFN
#' # 3                             Albanian Lek  ALL
#' # 4                            Armenian Dram  AMD
#' # 5            Netherlands Antillean Guilder  ANG
#' # 6                           Angolan Kwanza  AOA
#' # 7                           Argentine Peso  ARS
#' }
#'
#'


currencies <- function() {

  display_api_info()

  endpoint <- "http://api.exchangerate.host/live" %>%
    append_exchangeratehost_access_key

  live <- fromJSON(endpoint)

  df <- live$quotes %>%
    map_dbl(~ .x) %>%
    stack() %>%
    mutate(ind = substr(.data$ind, 4, nchar(as.character(.data$ind)))) %>%
    mutate(values = as.double(.data$values)) %>%
    .[,c(2,1)] %>%
    # Add USD since it won't be included
    add_row(ind = "USD", values = 1) %>%
    # then sort alphabetically
    arrange(.data$ind)

  df %>%
    select(.data$ind) %>%
    rename(code = .data$ind) %>%
    left_join(select(priceR::currency_info, .data$iso_code, .data$name), by = c("code" = "iso_code")) %>%
    select(description = .data$name, code = .data$code)

}















#'
#' Retrieve the latest exchange rates between the provided currency code
#' @name exchange_rate_latest
#'
#' @usage exchange_rate_latest(currency)
#'
#' @param currency A currency code (see currencies() for supported codes)
#'
#' @return A data.frame containing the latest exchange rates between the provided currency code
#'     and each other available currency
#'
#' @import dplyr purrr
#' @importFrom jsonlite fromJSON
#' @importFrom utils stack
#'
#' @export
#'
#' @examples
#' \dontrun{
#'
#' exchange_rate_latest("AUD")
#' # Daily AUD exchange rate as at end of day 2020-07-27 GMT
#' #     currency one_aud_is_equivalent_to
#' # 1        AED                  2.61894
#' # 2        AFN                 54.47724
#' # 3        ALL                 75.51799
#' # 4        AMD                343.40193
#' # 5        ANG                  1.26829
#' # 6        AOA                400.54604
#'
#'
#' # Defaults to USD
#' exchange_rate_latest()
#' #     currency one_USD_is_equivalent_to
#' # 1        AED                   3.6730
#' # 2        AFN                  76.4035
#' # 3        ALL                 105.9129
#' # 4        AMD                 481.6162
#' # 5        ANG                   1.7788
#' # 6        AOA                 561.7599
#'
#' # It can also accept other base rates
#' exchange_rate_latest("AUD")
#' #    currency one_AUD_is_equivalent_to
#' # 1       AED                  2.31619
#' # 2       AFN                 48.69229
#' # 3       ALL                 63.87806
#' # 4       AMD                260.72150
#' # 5       ANG                  1.13675
#' # 6       AOA                522.76772
#' }
#'

exchange_rate_latest <- function(currency = "USD") {

  currency = toupper(currency)

  display_api_info()

  endpoint <- "http://api.exchangerate.host/live" %>%
    append_exchangeratehost_access_key

  live <- fromJSON(endpoint)

  col_name <- paste0("one_", currency, "_is_equivalent_to")

  df <- live$quotes %>%
    map_dbl(~ .x) %>%
    stack %>%
    mutate(ind = substr(.data$ind, 4, (nchar(as.character(.data$ind))))) %>%
    mutate(values = as.double(.data$values)) %>%
    .[,c(2,1)] %>%
    # Add USD since it won't be included
    add_row(ind = "USD", values = 1) %>%
    # then sort alphabetically
    arrange(.data$ind)

  error_message = paste0("Currency \"", currency, "\" not available. Available currencies are:\n\n",
                         paste0(df$ind, collapse = (", ")))

  convert_currency <- function(df, currency) {
    # Extract conversion rate of the selected currency to USD
    usd_conversion_rate <- df %>%
      filter(.data$ind == currency) %>%
      pull(.data$values)

    # Compute new rates
    out <- df %>%
      mutate(values = ifelse(.data$ind == currency, 1, .data$values / usd_conversion_rate))

    return(out)
  }

  if(!currency %in% df$ind) { stop(error_message) }

  df_in_selected_currency <- convert_currency(df, currency)

  df_in_selected_currency <- df_in_selected_currency %>%
    `colnames<-`(c("currency", col_name))

  return(df_in_selected_currency)
}


















#'
#' Creates date ranges so as to batch up large API calls into many smaller ones
#' @name make_dates
#'
#' @usage make_dates(start_date, end_date, n_days)
#'
#' @param start_date A start date (of the form "2010-01-01")
#' @param end_date An end date
#' @param n_days The maximum number of days in each period
#'
#' @return A data.frame containing start and end dates for periods of length no longer than n_days
#'
#' @import dplyr
#'
#'
#' @examples
#'
#' # Simple test
#' start_date = "2010-01-01"
#' end_date = "2020-06-30"
#' n_days = 365
#' priceR:::make_dates(start_date, end_date, n_days)
#'
#' # With lots of periods
#' start_date = "2010-01-01"
#' end_date = "2020-06-30"
#' n_days = 20
#' priceR:::make_dates(start_date, end_date, n_days)
#'
#' # Less than one period
#' start_date = "2020-01-01"
#' end_date = "2020-06-30"
#' n_days = 365
#' priceR:::make_dates(start_date, end_date, n_days)
#'
#' # 366 days (note 2020 was a leap year)
#' start_date = "2019-07-30"
#' end_date = "2020-07-29"
#' n_days = 365
#' priceR:::make_dates(start_date, end_date, n_days)
#'
#' # 365 days
#' start_date = "2019-07-30"
#' end_date = "2020-07-28"
#' n_days = 365
#' priceR:::make_dates(start_date, end_date, n_days)
#'
#' # 1095 days (3 years)
#' start_date = "2019-07-30"
#' end_date = "2022-07-28"
#' n_days = 365
#' priceR:::make_dates(start_date, end_date, n_days)
#'


make_dates <- function(start_date, end_date, n_days) {

  # Helper for historical_exchange_rates, since API returns max 365 days at a time

  start_date <- as.Date(start_date)
  end_date <- as.Date(end_date)

  range_in_days <- (end_date - start_date) %>% as.integer %>%

    # Note that we add one since 2020-01-01 to 2020-01-02 is 2 days, yet one
    # minus the other gives a difference of only 1, so we add one to the difference

    `+`(1)

  # In cases where we don't need multiple periods,
  # simply return a 1 row data.frame

  if(range_in_days <= n_days) {
    dates <- data.frame(start_date=start_date,
                      end_date=end_date)
    return(dates)
  }


  from_ind <- seq(1, range_in_days, n_days)
  to_ind <- seq(n_days, range_in_days, n_days)

  # add the end date to the to_ind (except for cases where it's an exact multiple of n_days)
  if(range_in_days != last(to_ind)) {
    to_ind <- to_ind %>% c(range_in_days)
  }

  date_seq <- (start_date:end_date) %>% as.Date(origin = "1970-01-01")


  dates <- data.frame(start_date=date_seq[from_ind],
                      end_date=date_seq[to_ind])

  return(dates)
}



















#' Retrieve historical exchange rates
#'
#' Retrieves historical exchange rates between a currency pair - retrieves max. 365 days' data
#' @name retrieve_historical_rates
#'
#' @usage retrieve_historical_rates(from, to, start_date, end_date)
#'
#' @param from A currency code (see currencies() for supported codes)
#' @param to A currency code
#' @param start_date A start date (of the form "2010-01-01")
#' @param end_date An end date
#'
#'
#' @return A data.frame containing exchange rate data for select currency pair
#'
#' @import dplyr purrr
#' @importFrom jsonlite fromJSON
#' @importFrom utils stack
#'
#' @examples
#' \dontrun{
#' # Note date range >365 days', yet only 365 days' returned.
#' # Use historical_exchange_rates() for > 365 days'.
#'  priceR:::retrieve_historical_rates("USD", to = "AUD",
#'                                     start_date = "2018-01-01",
#'                                     end_date = "2020-06-30")
#' }
#'
#'


# # Some quick spot checks - recent data
# start_date <- "2022-01-01"
# end_date <- "2022-06-30"
#
# from = "AUD"
# to = "EUR"
#
# from = "AUD"
# to = "USD"
#
# from = "USD"
# to = "AUD"

# # Some quick spot checks old data (possibly missing data on weekends)
# # This short date rate will include some nulls for AUD to USD
# start_date <- "2010-01-01"
# end_date <- "2010-01-06"
#
# from = "AUD"
# to = "USD"


retrieve_historical_rates <- function(from, to, start_date, end_date) {

  # "http://api.exchangerate.host/timeframe?start_date=2015-01-01&end_date=2015-05-01&access_key=abcd1234"
  endpoint <- paste0("http://api.exchangerate.host/timeframe?start_date=",
                     start_date,
                     "&end_date=",
                     end_date,
                     "&currencies=",
                     paste0(c(from, to), collapse=",")
  ) %>% append_exchangeratehost_access_key

  dat <- endpoint %>%
    fromJSON

  col_name <- paste0("one_", from, "_equivalent_to_x_", to)

  num_rows <- dat[[8]] %>% length
  df <- data.frame(date = as.Date(rep(NA, num_rows)), values = as.numeric(rep(NA, num_rows)))


  df$date <- dat[[8]] %>% names

  # There are 3 possibilities to handle for
  # 1. Convert from USD to a non-USD currency.
  # 2. Convert from a non-USD currency to USD
  # 3. Convert between two non-USD currencies

  get_values <- function(response, index) {
    response[[8]] %>%
    purrr::map_dbl( ~ {.x[[index]] }) %>% unname
  }

  values <- if (from == "USD") {

  dat %>% get_values(1)

  } else if (to == "USD") {

  dat %>% get_values(1) %>% `/`(1, .)

  } else {
  # 3. Concert between two non-USD currencies

  one_usd_is_x_cur1 <- get_values(dat, 1)
  one_usd_is_x_cur2 <- get_values(dat, 2)
  one_usd_is_x_cur2 / one_usd_is_x_cur1

  }

  df$values <- values

  df <- df %>% `colnames<-`(c("date", col_name))
  df
}


















#' Retrieve historical exchange rates
#'
#' Retrieves historical exchange rates between a currency pair
#' @name historical_exchange_rates
#'
#' @usage historical_exchange_rates(from, to, start_date, end_date)
#'
#' @param from A currency code (see currencies() for supported codes)
#' @param to A currency code
#' @param start_date A start date (of the form "2010-01-01")
#' @param end_date An end date
#'
#'
#' @return A data.frame containing exchange rate data for select currency pair
#'
#' @import dplyr purrr
#' @importFrom jsonlite fromJSON
#'
#' @export
#'
#' @examples
#' \dontrun{
#' Retrieve AUD to USD exchange rates
#' au <- historical_exchange_rates(from = "AUD", to = "USD",
#'                           start_date = "2010-01-01", end_date = "2020-06-30")
#'
#' # Retrieve AUD to EUR exchange rates
#' ae <- historical_exchange_rates(from = "AUD", to = "EUR",
#'                           start_date = "2010-01-01", end_date = "2020-06-30")
#'
#' # Combine
#' cur <- au %>% left_join(ae, by = "date")
#'
#' head(cur)
#'}
#'
#'


historical_exchange_rates <- function(from, to, start_date, end_date) {

  # Validate currencies

  # Retrieved via: priceR::currencies()
  # Hard coded to avoid excessive API calls
  valid_currency_codes <- c(
    "AED", "AFN", "ALL", "AMD", "ANG", "AOA", "ARS", "AUD", "AWG",
    "AZN", "BAM", "BBD", "BDT", "BGN", "BHD", "BIF", "BMD", "BND",
    "BOB", "BRL", "BSD", "BTC", "BTN", "BWP", "BYN",

    "BYR", # Added due to two possible codes for Belarusian Ruble

    "BZD", "CAD",
    "CDF", "CHF", "CLF", "CLP", "CNH", "CNY", "COP", "CRC", "CUC",
    "CUP", "CVE", "CZK", "DJF", "DKK", "DOP", "DZD", "EGP", "ERN",
    "ETB", "EUR", "FJD", "FKP", "GBP", "GEL", "GGP", "GHS", "GIP",
    "GMD", "GNF", "GTQ", "GYD", "HKD", "HNL", "HRK", "HTG", "HUF",
    "IDR", "ILS", "IMP", "INR", "IQD", "IRR", "ISK", "JEP", "JMD",
    "JOD", "JPY", "KES", "KGS", "KHR", "KMF", "KPW", "KRW", "KWD",
    "KYD", "KZT", "LAK", "LBP", "LKR", "LRD", "LSL", "LYD", "MAD",
    "MDL", "MGA", "MKD", "MMK", "MNT", "MOP", "MRO", "MRU", "MUR",
    "MVR", "MWK", "MXN", "MYR", "MZN", "NAD", "NGN", "NIO", "NOK",
    "NPR", "NZD", "OMR", "PAB", "PEN", "PGK", "PHP", "PKR", "PLN",
    "PYG", "QAR", "RON", "RSD", "RUB", "RWF", "SAR", "SBD", "SCR",
    "SDG", "SEK", "SGD", "SHP", "SLL", "SOS", "SRD", "SSP", "STD",
    "STN", "SVC", "SYP", "SZL", "THB", "TJS", "TMT", "TND", "TOP",
    "TRY", "TTD", "TWD", "TZS", "UAH", "UGX", "USD", "UYU", "UZS",
    "VEF", "VES", "VND", "VUV", "WST", "XAF", "XAG", "XAU", "XCD",
    "XDR", "XOF", "XPD", "XPF", "XPT", "YER", "ZAR", "ZMW", "ZWL"
    )

  from = toupper(from)
  to = toupper(to)

  if (!all(c(from, to) %in% valid_currency_codes)){
    invalid_currencies <- c(from, to)[which(!c(from, to) %in% valid_currency_codes)]
    error_message = paste0("Invalid currency code(s): " ,"\"", paste0(invalid_currencies, collapse="\", \""), "\"",
". Run currencies()
  to view all ", length(valid_currency_codes), " valid currency codes.")
    stop(error_message)
  }

  display_api_info()

  api_time_splits <- make_dates(start_date, end_date, 365)

  # pmap*() variants map over rows of a data.frame
  pmap_dfr(api_time_splits, retrieve_historical_rates, from = from, to = to)


}












#'
#' Information for each of 191 currencies
#'
#'
#' @return A data.frame containing currency information for 191 currencies.
#'     Currency information includes: name, iso code, currency symbol (and
#'     alternative symbols if applicable), subunit, number of subunits per major
#'     unit, whether the currency symbol ought to appear before or after the
#'     number of units, display format, html entity, decimal mark,
#'     thousands separator, iso numeric, and smallest denomination.
#'
#'
#'
#' @examples
#' head(currency_info)

"currency_info"



### functions for `convert_currencies()`




#' Removes redundant API calls of currency pairs. That is, revmoes the need
#' to for separate calls for both
#' `from = EUR, to = USD` and `from = USD, to = EUR`
#'
#' @name pminmax
#'
#' @usage pminmax(x, y)
#'
#' @param x A currency code (see currencies() for supported codes)
#' @param y A currency code
#'
#' @return A character vector
#'
#'
#' @export
#'
#' @examples
#'
#' # See: https://stackoverflow.com/a/73254014/9059865
#'
#'
#'

pminmax <- function(x, y) {
  paste(pmin.int(x, y), pmax.int(x, y), sep = ".")
}

#' Wrapper around `priceR::historical_exchange_rates()` with slight modifications
#' to structure of inputs and output
#'
#' @name from_to_dates_rates
#'
#' @usage from_to_dates_rates(from, to, dates)
#'
#' @param from A currency code (see currencies() for supported codes)
#' @param to A currency code
#' @param dates A list of date ranges
#'
#' @return A data.frame with two columns: date (of class Date), and rate
#'     (of class numeric).
#'
#'
#' @export
#'
#' @examples
#' \dontrun{
#' library(lubridate)
#' from_to_dates_rates("AUD", "USD", dates = list(today()-10, today()))
#' }
#'

from_to_dates_rates <- function(from, to, dates) {
  historical_exchange_rates(
    from = from,
    to = to,
    start_date = dates[[1]],
    end_date = dates[[2]]
  ) %>%
    purrr::set_names("date", "rate")
}

#' Convert Currencies
#'
#' Vectorized approach to converting prices across potentially different dates
#' and between different currencies.
#'
#' @param price_start Numeric vector of prices in terms of `from` currenc(ies).
#' @param from Character vector of currenc(ies) of `price_start`.
#' @param to Character vector of currenc(ies) to convert `price_start` to.
#' @param date Date vector specifying date of exchange rate to use.
#' @param floor_unit Character string. Default is "day" meaning that `date` will
#'   be converted based on daily conversion rates. Changing to "week" will
#'   change conversions to be based on the start of the week of `date`.
#'
#' @return Numeric vector of `price_start` now in the `to` currenc(ies).
#' @export
#'
#' @examples
#' \dontrun{
#' library(dplyr)
#'
#' sales_transactions <- tibble(
#'   local_price = c(100, 80, 9200, 90),
#'   local_currency = c("USD", "EUR", "JPY", "USD"),
#'   final_currency = c("EUR", "USD", "USD", "JPY"),
#'   date_transaction = lubridate::ymd(c(20200601, 20200609, 20200614, 20200623))
#' )
#' # Some made-up sales transactions of different values and currencies
#' sales_transactions %>%
#'   mutate(
#'     converted_price = convert_currencies(
#'       price_start = local_price,
#'       from = local_currency,
#'       to = final_currency,
#'       date = date_transaction
#'     )
#'   )
#' }
convert_currencies <- function(price_start,
                               from,
                               to,
                               date = lubridate::today(),
                               floor_unit = "day") {

  # Note: https://stackoverflow.com/a/8096882/5783745 silences the following
  # NOTE in the R CMD check --as-cran
  # convert_currencies: no visible binding for global variable ‘from_to’
  # convert_currencies: no visible binding for global variable ‘date_range’
  # convert_currencies: no visible binding for global variable ‘rate’
  # convert_currencies: no visible binding for global variable ‘price’
  # convert_currencies: no visible binding for global variable ‘output’
  from_to = date_range = rate = price = output = NULL

  rates_start <- dplyr::tibble(
    from = from,
    to = to,
    date = date %>%
      as.Date() %>%
      lubridate::floor_date(floor_unit)
  ) %>%
    dplyr::mutate(from_to = pminmax(from, to)) %>%
    dplyr::distinct(from_to, date, .keep_all = TRUE)

  # When passing things to the priceR API it is much faster to send over a range
  # of dates rather than doing individual API calls for each date (even when
  # there are many redundant dates between `from` and `to` values).

  rates_end <- rates_start %>%
    dplyr::group_by(from_to) %>%
    dplyr::summarise(
      date_range = list(range(date)),
      from = from[[1]],
      to = to[[1]],
      rates_lookup = purrr::pmap(.l = list(from, to, date_range),
                          .f = from_to_dates_rates)
    ) %>%
    dplyr::select(-date_range) %>%
    tidyr::unnest(rates_lookup)

  rates_lookup <- rates_end %>%
    mutate(date = lubridate::ymd(date)) %>%
    dplyr::semi_join(rates_start, "date")

  # this step makes it so could convert either "from" or "to" currency
  rates_lookup <- dplyr::bind_rows(
    rates_lookup,
    rates_lookup %>%
      dplyr::rename(from = to, to = from) %>%
      dplyr::mutate(rate = 1 / rate)
  ) %>%
    dplyr::distinct()

  dplyr::tibble(
    price = price_start,
    from = from,
    to = to,
    date = date
  ) %>%
    dplyr::left_join(rates_lookup, c("from", "to", "date")) %>%
    dplyr::mutate(output = price * rate) %>%
    dplyr::pull(output)
}

#' Extract numeric salary from text data
#'
#' Extract numeric salary from text data. `extract_salary` automatically converts weekly and hourly rates to amounts per annum.
#' @name extract_salary
#'
#' @usage extract_salary(salary_text, exclude_below, exclude_above, salary_range_handling,
#' include_periodicity, hours_per_workday, days_per_workweek, working_weeks_per_year)
#'
#' @param salary_text A character string, or vector of character strings.
#' @param exclude_below A lower bound. Anything lower than this number will be replaced with NA.
#' @param exclude_above An upper bound. Anything above this number will be replaced with NA.
#' @param salary_range_handling A method of handling salary ranges. Defaults to returning an average of the range; can also be set to "max" or "min".
#' @param include_periodicity Set to TRUE to return an additional column stating the detected peridicity in the character string. Periodicity is assumed to be 'Annual' unless evidence is found to the contrary.
#' @param hours_per_workday Set assumed number of hours in the workday. Only affects annualisation of rates indentified as Daily. Default is 8 hours.
#' @param days_per_workweek Set assumed number of days per workweek. Only affects annualisation of rates indentified as Daily. Default is 5 days.
#' @param working_weeks_per_year Set assumed number of working weeks in the year. Only affects annualisation of rates indentified as Daily or Weekly. Default is 50 weeks.
#'
#' @return A data.frame of 1 column, or 2 columns if include_periodicity is set to TRUE
#'
#' @import dplyr stringr gsubfn
#'
#' @export
#'
#' @examples
#'
#' # Provide a salary string and 'extract_salary' and will extract the salary and return it
#' extract_salary("$160,000 per annum")
#' # 160000
#'
#'
#' # If a range is present, the average will be taken by default
#' extract_salary("$160,000 - $180000.00 per annum")
#' # 170000
#'
#'
#' # Take the 'min' or 'max' of a salary range by setting salary_range_handling parameter accordingly
#' extract_salary("$160,000 - $180000.00 per annum", salary_range_handling = "min")
#' # 160000
#'
#'
#' # Extract salaries from character string(s)
#' annual_salaries <- c("$160,000 - $180000.00 per annum",
#'                      "$160000.00 - $180000.00 per annum",
#'                      "$145000 - $155000.00 per annum",
#'                      "$70000.00 - $90000 per annum",
#'                      "$70000.00 - $90000.00 per annum plus 15.4% super",
#'                      "$80000.00 per annum plus 15.4% super",
#'                      "60,000 - 80,000",
#'                      "$78,686 to $89,463 pa, plus 15.4% superannuation",
#'                      "80k - 100k")
#'
#' extract_salary(annual_salaries)
#' # 170000 170000 150000  80000  53338  40008  70000  56055  90000
#' # Note the fifth, sixth, and eighth elements are averages including '15' (undesirable)
#' # Using exclude_below parameter avoids this (see below)
#'
#' # Automatically detect, extract, and annualise daily rates
#' daily_rates <- c("$200 daily", "$400 - $600 per day", "Day rate negotiable dependent on experience")
#' extract_salary(daily_rates)
#' # 48000 120000     NA
#'
#'
#' # Automatically detect, extract, and annualise hourly rates
#' hourly_rates <- c("$80 - $100+ per hour", "APS6/EL1 hourly rate contract")
#' extract_salary(hourly_rates)
#' # 172800   6720
#' # Note 6720 is undesirable. Setting the exclude_below and exclude_above sensibly avoids this
#'
#'
#' salaries <- c(annual_salaries, daily_rates, hourly_rates)
#'
#'
#' # Setting lower and upper bounds provides a catch-all to remove unrealistic results
#' # Out of bounds values will be converted to NA
#' extract_salary(salaries, exclude_below = 20000, exclude_above = 600000)
#' # 170000 170000 150000  80000  80000  80000  70000  84074  90000  48000 120000     NA 172800     NA
#'
#'
#' # extract_salary automatically annualises hourly and daily rates
#' # It does so by making assumptions about the number of working weeks in a year,
#' # days per workweek, and hours per workday
#' # And the assumed number of hours per workday can be changed from the default (8)
#' # The assumed number of workdays per workweek can be changed from the default (5)
#' # The assumed number of working weeks in year can be changed from the default (50)
#' # E.g.
#' extract_salary(salaries, hours_per_workday = 7, days_per_workweek = 4,
#'                working_weeks_per_year = 46, exclude_below = 20000)
#' # 170000 170000 150000  80000  53338  40008  70000  56055  90000  36800  92000     NA 115920     NA
#'
#'
#' # To see which salaries were detected as hourly or weekly, set include_periodicity to TRUE
#' extract_salary(salaries, include_periodicity = TRUE, exclude_below = 20000)
#'
#' # salary periodicity
#' # 1  170000      Annual
#' # 2  170000      Annual
#' # 3  150000      Annual
#' # 4   80000      Annual
#' # 5   80000      Annual
#' # 6   80000      Annual
#' # 7   70000      Annual
#' # 8   84074      Annual
#' # 9   90000      Annual
#' # 10  48000       Daily
#' # 11 120000       Daily
#' # 12     NA       Daily
#' # 13 172800      Hourly
#' # 14     NA      Hourly
#'
#'


extract_salary <- function(salary_text, exclude_below, exclude_above, salary_range_handling, include_periodicity, hours_per_workday, days_per_workweek, working_weeks_per_year) {


  ### These steps ensure any digits found via regular expressions are replaced with complete numbers
  # and not scientific notation! - very important.

  # Set existing option(s)
  op <- options("scipen")

  # Give scipen option a new value
  options(scipen = 100)

  # Run this to ensure that on existing the current scope (i.e. function), the option is set back to
  # its original value
  on.exit(options(op))





  if(missing(exclude_above)) { exclude_above <- 9999999999999 }
  if(missing(exclude_below)) { exclude_below <- 0 }
  if(missing(include_periodicity)) { include_periodicity <- FALSE }

  if(missing(hours_per_workday)) { hours_per_workday <- 8 }
  if(missing(days_per_workweek)) { days_per_workweek <- 5 }
  if(missing(working_weeks_per_year)) { working_weeks_per_year <- 50 }

  if(missing(salary_range_handling)) { salary_range_handling <- "average" }
  if(!salary_range_handling %in% c("average", "max", "min")) { stop("salary_range_handling parameter must be either \"average\", \"min\", or \"max\"") }

  # Clean
  salary <- salary_text %>% gsub(",", "", .) %>% str_replace_all(., "\\d+\\.\\d+", function(x) { as.character(as.integer(x)) }) %>%
    gsub("(\\d+)k", "\\1000", .)  %>% gsub("(\\d+)K", "\\1000", .) # Sub out commas, round decimals, and convert k to 000


  # Identify hourly, daily or annual
  hourly_daily_or_annual <- function(salary) {

    hourly_terms <- c("ph", "p\\.h", "p\\.h\\.", "p/h", "hourly", "hour")
    daily_terms <- c("pd", "p\\.d", "p\\.d\\.", "p/d", "daily", "day")

    period <- salary %>% { ifelse((grepl(paste(daily_terms, collapse="|"), . , ignore.case=TRUE)), "Daily",
                                  ifelse(grepl(paste(hourly_terms, collapse="|"), . , ignore.case=TRUE), "Hourly",
                                         "Annual")) }
    temp <- data.frame(salary, period, stringsAsFactors = FALSE)
    return(temp)
  }
  temp <- hourly_daily_or_annual(salary)


  # For every number regex'd out, these will convert into annual
  hourly_to_annual <- function(hourly_pay) {
    hourly_pay %>% as.numeric %>% {. * hours_per_workday * days_per_workweek * working_weeks_per_year} %>%
      round %>% { ifelse(. > exclude_above, "", .) } %>% { ifelse(. < exclude_below, "", .) }
  }

  salary[temp$period == "Hourly"] <- salary[temp$period == "Hourly"] %>% gsubfn("(\\d+)", ~ { hourly_to_annual(x) }, . , backref = -1) %>% unlist

  daily_to_annual <- function(daily_pay) {
    daily_pay %>% as.numeric %>% {. * days_per_workweek * working_weeks_per_year} %>%
      round %>% { ifelse(. > exclude_above, "", .) } %>% { ifelse(. < exclude_below, "", .) }
  }

  salary[temp$period == "Daily"] <- salary[temp$period == "Daily"]  %>% gsubfn("(\\d+)", ~ { daily_to_annual(x) }, . , backref = -1) %>% unlist


  # Define 3 functions
  extract_average <- function(salary) {
    salary %>% str_extract_all(. , "\\d+") %>% lapply(., function(x) { x[as.numeric(x) >= exclude_below] } ) %>%
      lapply(., function(x) { x[as.numeric(x) <= exclude_above] } ) %>% lapply(., function(x) { mean(as.numeric(x)) } ) %>% unlist %>% {ifelse(. == -Inf, NA_character_, .) }
  }

  extract_max <- function(salary) {
    salary %>% str_extract_all(. , "\\d+") %>% lapply(., function(x) { x[as.numeric(x) >= exclude_below] } ) %>%
      lapply(., function(x) { x[as.numeric(x) <= exclude_above] } )%>% lapply(., function(x) { max(as.numeric(x)) } ) %>% unlist %>% {ifelse(. == -Inf, NA_character_, .) }
  }

  extract_min <- function(salary) {
    salary %>% str_extract_all(. , "\\d+") %>% lapply(., function(x) { x[as.numeric(x) >= exclude_below] } ) %>%
      lapply(., function(x) { x[as.numeric(x) <= exclude_above] } ) %>% lapply(., function(x) { min(as.numeric(x)) } ) %>% unlist %>% {ifelse(. == -Inf, NA_character_, .) }
  }


  switch(salary_range_handling,
         "average"=suppressWarnings({salary <- salary %>% extract_average %>% as.numeric}),
         "min"=suppressWarnings({salary <- salary %>% extract_min %>% as.numeric}),
         "max"=suppressWarnings({salary <- salary %>% extract_max %>% as.numeric})
  )

  salary <- salary %>% { ifelse(. > exclude_above, "", .) } %>% as.numeric(.) %>% { ifelse(. < exclude_below, "", .) } %>% as.numeric(.) %>% round

  annualised_salaries <- data.frame(salary, stringsAsFactors = FALSE)

  if(include_periodicity) {
    annualised_salaries <- data.frame(annualised_salaries, temp$period, stringsAsFactors = FALSE)
    colnames(annualised_salaries)[2] <- "periodicity"
  }


  return(annualised_salaries)
}

#' Make numeric currency values human readable
#'
#' @name format_currency
#'
#' @usage format_currency(amount, symbol, digits)
#'
#' @param amount Price or vector of prices (character, numeric, or integer)
#' @param symbol Symbol to prepend to amount (e.g. $) see: currency_characters()
#' @param digits The number of decimal places. Set equal to 2 to include cents (defaults to 0 i.e. whole major currency units)
#'
#' @return A character vector
#'
#' @import dplyr stringr
#'
#' @export
#'
#' @examples
#'
#' # format_currency("2423562534234", "$")
#' # "$2,423,562,534,234"
#'
#' # format_currency("2423562534234.876", "$", 0)
#' # "$2,423,562,534,234"
#'
#' # format_currency("2423562534234.876", "$", 2)
#' # "$2,423,562,534,234.88"
#'
#' # format_currency("2423562534234", "¥", 2)
#' # "¥2,423,562,534,234.00"
#'
#' # format_currency() is vectorized and can accept vector arguments
#' format_currency(c("2423562534234", "20"), c("¥", "$"), c(1, 2))
#' # "¥2,423,562,534,234.0" "$20.00"
#'


format_currency <- function(amount, symbol, digits) {

  if(missing(digits)) { digits <- 0 }

  single_format <- function(amount, symbol, digits) {

    # From: https://stackoverflow.com/questions/14836754/is-there-an-r-function-to-escape-a-string-for-regex-characters
    symbol_regex <- gsub("([.|()\\^{}+$*?]|\\[|\\])", "\\\\\\1", symbol)

    formatted <- amount %>%
      gsub(",", "", .) %>%
      as.numeric %>%
      round(., digits) %>%
      formatC(format="f", digits=digits, big.mark=",", small.mark = digits) %>%
      trimws %>%
      paste0(symbol, .)

    return(formatted)
  }

  formatted_vector <- mapply(single_format, amount, symbol, digits, SIMPLIFY = TRUE) %>%
    unname

  return(formatted_vector)
}








#' Make numeric currency values human readable
#'
#' @name format_dollars
#'
#' @usage format_dollars(amount, digits)
#'
#' @param amount Price or vector of prices (character, numeric, or integer)
#' @param digits The number of decimal places. Set equal to 2 to include cents (defaults to 0 i.e. whole dollars)
#'
#' @return A character vector
#'
#' @import dplyr
#' @export
#'
#' @examples
#'
#' # format_dollars("2423562534234")
#' # "$2,423,562,534,234"
#'
#' # format_dollars("2423562534234.876", 0)
#' # "$2,423,562,534,234"
#'
#' # format_dollars("2423562534234.876", 2)
#' # "$2,423,562,534,234.88"
#'
#' # format_dollars("2423562534234", 2)
#' # "$2,423,562,534,234.00"

format_dollars <- function(amount, digits) {

  format_currency(amount, "$", digits)

}











#' Provide currency characters
#'
#' @name currency_characters
#'
#' @usage currency_characters()
#'
#' @return A character vector of currency symbols
#'
#' @import stringi
#' @export
#'
#' @examples
#'
#' currency_characters()
#'


currency_characters <- function() {

  c("\\u0024", "\\u00a4", "\\u00a3", "\\u20ac", "\\u00a2", "\\u00a5", "\\u20a7",
    "\\u0192") %>% stringi::stri_unescape_unicode()

}








#' Convert human readable currencies into numeric data
#'
#' @name currency_to_numeric
#'
#' @usage currency_to_numeric(currency_text)
#'
#' @param currency_text Price or vector of prices
#'
#' @return A numeric vector
#'
#' @import dplyr
#' @export
#'
#' @examples
#' library(dplyr)
#' c("$134,345.05", "£22", "¥30000") %>% currency_to_numeric()
#' # [1] 134345     22  30000


currency_to_numeric <- function(currency_text) {

  currency_symbols <- currency_characters()

  currency_symbols_regex <- currency_symbols %>% gsub("\\$", "\\\\$", .)

  currency_text %>%
    { gsub(paste0(currency_symbols_regex, collapse="|"), "", .) } %>%
    gsub(",", "", .) %>%
    as.numeric

}

context("Formatting functions work as expected")

# devtools::test()



#----- format_currency -----#

# test_that("format_currency() correctly inserts commas and symbol to integer value", {
#
#   expect_equal(format_currency("2423562534234", "$"), "$2,423,562,534,234")
#
# })
#
#
#
# test_that("format_currency() correctly inserts commas and symbol to decimal value and
#           rounds to zero decimal places", {
#
#   expect_equal(format_currency("2423562534234.876", "$", 0), "$2,423,562,534,235")
#
# })
#
#
# test_that("format_currency() correctly inserts commas and symbol other than dollar sign", {
#
#   expect_equal(format_currency("2423562534234", "¥", 2), "¥2,423,562,534,234.00")
#
# })
#
#

# test_that("format_currency() correctly handles vector arguments", {
#
#   expect_equal(format_currency(c("2423562534234", "20"), c("¥", "$"), c(1, 2)),
#                c("¥2,423,562,534,234.0", "$20.0"))
#
# })










#----- format_dollars -----#

# test_that("format_dollars() correctly inserts commas and symbol to integer value", {
#
#   expect_equal(format_dollars("2423562534234"), "$2,423,562,534,234")
#
# })
#
#
# test_that("format_dollars() correctly inserts commas and symbol to decimal value and
#           rounds to zero decimal places", {
#
#             expect_equal(format_dollars("2423562534234.876", 0), "$2,423,562,534,235")
#
#           })
#
#
# test_that("format_dollars() correctly inserts commas and symbol and rounds to 2dp", {
#
#   expect_equal(format_dollars("2423562534234.876", 2), "$2,423,562,534,234.88")
#
# })
#
#
#
# test_that("format_dollars() correctly inserts commas and symbol and rounds to 2dp when those
#           2dp are both zeros", {
#
#   expect_equal(format_dollars("2423562534234", 2), "$2,423,562,534,234.00")
#
# })
#

#' @description priceR: Economics and Pricing Tools
#' @docType package
#'
#' @details
#' It has the goal of providing a quick and practical way of extracting numeric price and currency data from text fields, thus allowing faster and easier analysis of ostensibly numeric data.
#'
#'
#' @author Steve Condylios \email{steve.condylios@gmail.com}
#'
#'
#'
#' @name priceR-package
#'
# NULL
# From jennybc's comment here: https://github.com/STAT545-UBC/Discussion/issues/451
# quiets concerns of R CMD check re: the .'s that appear in pipelines
if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))

#' Generate a World Bank API URL that will return all results for a given indicator in JSON format
#'
#' results and returns JSON format
#'
#' @name url_all_results
#'
#' @usage url_all_results(original_url)
#'
#' @param original_url A World Bank API URL. E.g. "https://api.worldbank.org/v2/country".
#'
#' @return A character vector
#'
#' @import dplyr stringr
#'
#' @export
#'
#' @examples
#'
#' # Provide a World Bank API URL and `url_all_results` will convert it into one with all results
#' # for that indicator
#' \dontrun{
#'   original_url <- "https://api.worldbank.org/v2/country" # Note: no ?format=json on url
#'   url_all_results(original_url)
#' }
#' # "https://api.worldbank.org/v2/country?format=json&per_page=304"
#'
#'
#'



url_all_results <- function(original_url) {

# Append "?format=json" if url doesn't already have it
  if(!grepl("\\?", original_url)) {

    original_url <- paste0(original_url, "?format=json")

  }

  total_results <- original_url %>% fromJSON %>% .[[1]] %>% .$total
  cat("Generating URL to request all", total_results, "results\n")
  url_with_all_results <- paste0(original_url, "&per_page=", total_results)

  url_with_all_results

}








#'  Show available country codes
#'
#' `show_countries` calls the World Bank API and retrieves a list of available countries and regions
#' @name show_countries
#'
#' @usage show_countries()
#'
#' @return A data.frame of countries available to query using the World Bank API
#'
#' @import dplyr stringr
#' @importFrom jsonlite fromJSON
#'
#' @export
#'
#' @examples
#'
#' # Simply call show_countries() to receive a dataframe of all countries (and regions) and their
#' # iso2Code
#'
#' # show_countries()
#' #       iso2Code         country_name
#' #   1         AW                Aruba
#' #   2         AF          Afghanistan
#' #   3         A9               Africa
#' #   4         AO               Angola
#' #   Etc
#'
#'

show_countries <- function() {

  countries_url <- "https://api.worldbank.org/v2/country?format=json" %>% url_all_results(.)
  countries <- countries_url %>% fromJSON(.)

  countries <- data.frame(countries[[2]]$iso2Code, countries[[2]]$name, stringsAsFactors = FALSE)
  colnames(countries) <- c("iso2Code", "country_name")

  return(countries)
}






#' Determines whether country input is a country name or iso2Code
#'
#' Determines whether a string is a country name, an iso2Code, or invalid (not a World Bank API
#' country/region)
#' @name country_input_type
#'
#' @usage country_input_type(country_input, countries_dataframe)
#'
#' @param country_input A country/region the user wishes to validate (string) E.g. "Australia".
#' @param countries_dataframe A dataframe containing available iso2Code and country_name
#' (see show_countries()).
#'
#' @return A character vector
#'
#' @import dplyr stringr
#' @importFrom jsonlite fromJSON
#'
#' @export
#'
#' @examples
#'
#' \dontrun{
#' # Assign so as to save on API calls - recommended
#' countries_dataframe <- show_countries()
#'
#'   country <- "Australia"
#'   country_input_type(country, countries_dataframe)
#' # [1] "country_name"
#'
#'   country <- "AU"
#'   country_input_type(country, countries_dataframe)
#' # [1] "iso2Code"
#'
#'   country <- "something other than a valid country name or iso2Code"
#'   country_input_type(country, countries_dataframe)
#' # [1] "invalid"
#' }

country_input_type <- function(country_input, countries_dataframe) {

  # Logic: if the country_input is found among the iso2Codes, assume it's an iso2Code,
  # if the country_input if not found in iso2Codes, then check to see if it's found among
  # country_names,
  # and if not, return 'invalid'

  if (which(countries_dataframe$iso2Code %in% country_input) %>% length > 0) {
    country_input_type_string <- "iso2Code"
  } else if (which(countries_dataframe$country_name %in% country_input) %>% length > 0) {
    country_input_type_string <- "country_name"
  } else {
    country_input_type_string <- "invalid"
    }
  return(country_input_type_string)
}



























#' Convert any country input into its iso2Code
#'
#' `convert_to_iso2Code` accepts the type of country input and the country, and returns the relevant iso2Code
#' @name convert_to_iso2Code
#'
#' @usage convert_to_iso2Code(country_input_type_string, country, countries_dataframe)
#'
#' @param country_input_type_string Either "country_name" or "iso2Code" - use country_input_type(country, countries_dataframe) to determine or assign manually.
#' @param country A country/region name or iso2Code.
#' @param countries_dataframe The output of show_countries()
#'
#' @return A character vector containing a valid iso2Code
#'
#' @import dplyr stringr
#' @importFrom jsonlite fromJSON
#'
#' @export
#'
#' @examples
#'
#' \dontrun{
#'
#' # Assign so as to save on API calls (recommended)
#' countries_dataframe <- show_countries()
#' country <- "Australia"
#' country_input_type_string <- country_input_type(country, countries_dataframe)
#' convert_to_iso2Code(country_input_type_string, country, countries_dataframe)
#' # [1] "AU"
#'
#' country <- "AU"
#' country_input_type_string <- country_input_type(country, countries_dataframe)
#' convert_to_iso2Code(country_input_type_string, country, countries_dataframe)
#' # [1] "AU"
#'
#'
#' }
#'

convert_to_iso2Code <- function(country_input_type_string, country, countries_dataframe) {

  if(country_input_type_string == "iso2Code") { country <- country }

  if(country_input_type_string == "invalid") { stop(paste0("'", country, "'", " is not a valid country input - select a valid country from show_countries()")) }

  if(country_input_type_string == "country_name") {

    index_of_country_in_countries <- which(countries_dataframe$country_name %in% country)
    country <- countries_dataframe$iso2Code[index_of_country_in_countries]
  }

  return(country)
}



























#' Retrieve historical inflation data
#'
#' Retrieve inflation data for any country/region (using iso2Code or country_name)
#' @name retrieve_inflation_data
#'
#' @usage retrieve_inflation_data(country, countries_dataframe)
#'
#' @param country A country_name or iso2code (see show_countries() for complete list of available inputs).
#' @param countries_dataframe The output from show_countries(). It is optional, but if not provided, it will be retrieved via the API.
#'
#' @return A data.frame containing inflation data from World Bank API for specified country
#'
#' @import dplyr stringr
#' @importFrom jsonlite fromJSON
#'
#' @export
#'
#' @examples
#' \dontrun{
#' # Retrieve inflation data for any country (or iso2Code)
#' country <- "AU"
#' inflation_dataframe <- retrieve_inflation_data(country)
#'
#' country <- "Australia"
#' countries_dataframe <- show_countries()
#' inflation_dataframe <- retrieve_inflation_data(country, countries_dataframe)
#' }
#' # inflation_dataframe
#' #    indicator.id                       indicator.value country.id country.value     value
#' #  FP.CPI.TOTL.ZG Inflation, consumer prices (annual %)         AU     Australia      <NA>
#' #  FP.CPI.TOTL.ZG Inflation, consumer prices (annual %)         AU     Australia   1.94864
#' #  FP.CPI.TOTL.ZG Inflation, consumer prices (annual %)         AU     Australia   1.27699
#' #  FP.CPI.TOTL.ZG Inflation, consumer prices (annual %)         AU     Australia   1.50836
#' #  Etc
#'
#'


retrieve_inflation_data <- function(country, countries_dataframe) {

  if(missing(countries_dataframe)) {
    cat("Validating iso2Code for", country, "\n")
    countries_dataframe <- show_countries()
    }

  # Ensure we have an iso2Code
  country_input_type_string <- country_input_type(country, countries_dataframe)
  country <- convert_to_iso2Code(country_input_type_string, country, countries_dataframe)

  cat("Retrieving inflation data for", country, "\n")

  # Works
  # https://api.worldbank.org/v2/country/AU/indicator/FP.CPI.TOTL.ZG?format=json

  inflation_url <- paste0("https://api.worldbank.org/v2/country/", country, "/indicator/FP.CPI.TOTL.ZG")

  inflation_url <- inflation_url %>% url_all_results

  inflation_data <- inflation_url %>% fromJSON(.)
  inflation_data <- inflation_data[[2]]

  return(inflation_data)
}












#' Convert nominal prices into real prices
#'
#' Inflate/deflate prices from any year to any year, using World Bank inflation data and assumptions only where necessary.
#' Typically used for converting past (nominal) values into current (real) values. This uses World Bank inflation data where available,
#' but allows for both historical and future assumptions in extrapolation.
#' @name adjust_for_inflation
#'
#' @usage adjust_for_inflation(price, from_date, country, to_date, inflation_dataframe,
#' countries_dataframe, extrapolate_future_method, future_averaging_period, future_rate,
#' extrapolate_past_method, past_averaging_period, past_rate)
#'
#' @param price A price (or prices).
#' @param from_date A date(s) from which the prices will be converted.
#' @param country A country or region in whose currency the prices are denominated.
#' @param to_date A date(s) to which the prices will be converted.
#' @param inflation_dataframe The R object (list) representing the JSON retrieved by calling retrieve_inflation_data().
#' @param countries_dataframe The R object (data.frame) representing the JSON retreived by calling show_countries().
#' @param extrapolate_future_method The extrapolation method that shall be used if extrapolation into the future is required. Options are 'average' or 'rate'.
#' @param future_averaging_period The number of recent periods to average in order to extrapolate forward (if 'average' is method being used).
#' @param future_rate An assumed rate of inflation to use for extrapolating forward (if 'rate' is method being used).
#' @param extrapolate_past_method The extrapolation method that shall be used if extrapolation from the earliest available data to some even earlier period is required.
#' @param past_averaging_period The number of periods back from the earliest available inflation data for a given country to average in order to extrapolate into the past (if 'average' is method being used).
#' @param past_rate An assumed rate of inflation to use for extrapolating from the earliest available data to some even earlier period (if 'rate' is method being used).
#'
#' @return A vector of inflation-adjusted prices
#'
#' @import dplyr stringr
#' @importFrom jsonlite fromJSON
#' @importFrom purrr map_df
#' @importFrom stats na.omit
#'
#' @export
#'
#'
#' @examples
#' \dontrun{
#' # Assign these variables once
#' country <- "AU"
#' inflation_dataframe <- retrieve_inflation_data(country)
#' countries_dataframe <- show_countries()
#'
#' # Convert $100 from 2005 into 2017 dollars
#'
#' adjust_for_inflation(100, 2005, country, to_date = 2017,
#' inflation_dataframe = inflation_dataframe,
#' countries_dataframe = countries_dataframe)
#'
#' # [1] 133.9861 # i.e. $100 in 2005 had the same purchasing power as $133.99 in 2017
#' }
#'






adjust_for_inflation <- function(price,
                                 from_date,
                                 country,
                                 to_date,
                                 inflation_dataframe,
                                 countries_dataframe,
                                 extrapolate_future_method, # exf_method
                                 future_averaging_period, #exf_period
                                 future_rate, # exf_rate
                                 extrapolate_past_method, #exp_method
                                 past_averaging_period, # exp_period
                                 past_rate) { # exp_rate

  # Later, it would be great to include a parameter for 'extrapolate = TRUE' - this could project for earlier and later dates, rather than returning NA


  #----- Deal with missing parameters and define functions -----#

  if(missing(extrapolate_future_method)) { extrapolate_future <- FALSE } else { extrapolate_future <- TRUE }
  if(missing(extrapolate_past_method)) { extrapolate_past <- FALSE } else { extrapolate_past <- TRUE }

  if(extrapolate_future) {
    if(missing(extrapolate_future_method)) { stop("'extrapolate_future_method' must be specified (it can take values 'average' or 'rate')") }
    if(!extrapolate_future_method %in% c("average", "rate")) { stop("'extrapolate_future_method' must be either 'average' or 'rate'")}
    if(extrapolate_future_method == 'average') { if(missing(future_averaging_period)) {
      stop("Please specify how many years' average to use when extrapolating forward ('future_averaging_period'
           can take any positive integer or 'all' to use an average of all available years' data")}}
    if(extrapolate_future_method == 'rate') {
      if(missing(future_rate)) { stop("Please specify the assumed rate of inflation for future periods (i.e. using 'future_rate' parameter)")}
      }

  }

  if(extrapolate_past) {
    if(missing(extrapolate_past_method)) { stop("'extrapolate_past_method' must be specified (it can take values 'average' or 'rate')") }
    if(!extrapolate_past_method %in% c("average", "rate")) { stop("'extrapolate_past_method' must be either 'average' or 'rate'")}
    if(extrapolate_past_method == 'average') { if(missing(past_averaging_period)) {
    stop("Please specify how many years' average to use when extrapolating forward ('past_averaging_period'
         can take any positive integer or 'all' to use an average of all available years' data")}}
    if(extrapolate_past_method == 'rate') {
      if(missing(past_rate)) { stop("Please specify the assumed rate of inflation for past periods (i.e. using 'past_rate' parameter)")}
    }
  }


  # If no to_date is provided, assume conversion into present day dollars is intended
  if(missing(to_date)) {
    to_date <- rep(Sys.Date(), length(price)) %>% substr(., 1, 4) %>% as.integer
  } else { to_date <- to_date %>% substr(., 1, 4) %>% as.integer }


  from_date <- substr(from_date, 1, 4) %>% as.integer # Ensure to / from are just years (for now)
  to_date <- substr(to_date, 1, 4) %>% as.integer # Ensure to / from are just years (for now)


  # Validate that there are as many dates as prices (or just one date)
  if(!(length(price) == length(from_date) | length(from_date) == 1)) {
    stop("from_date must be a date or a vector of dates of the same length as price")
  }

  if(missing(countries_dataframe)) {
  message("Retrieving countries data")
  # Determine country input type
  countries_dataframe <- show_countries()
  }

  country_input_type_string <- country_input_type(country, countries_dataframe)
  country <- convert_to_iso2Code(country_input_type_string, country, countries_dataframe)
  # 'country' is iso2Code from here on

  name_of_country <- which(countries_dataframe$iso2Code %in% country) %>% countries_dataframe$country_name[.]


  if(missing(inflation_dataframe)) { inflation_dataframe <- retrieve_inflation_data(country, countries_dataframe = countries_dataframe) }
  inflation_dataframe <- inflation_dataframe %>% .[ , c("value", "date")]
  inflation_dataframe$date <- inflation_dataframe$date %>% as.integer
  inflation_dataframe$value <- inflation_dataframe$value %>% as.numeric



  #----- Extrapolation logic -----#

  # Extrapolating future
  available_inflation_data <- inflation_dataframe %>% na.omit
  max_year_requested <- max(c(to_date, from_date)) # from_date included here for edge case where
  # going from future from date to future to_date AND where from_date > to_date
  max_year_available_without_extrapolation <- max(available_inflation_data$date)

  if(extrapolate_future & (max_year_requested > max_year_available_without_extrapolation)) {

    if(extrapolate_future_method == 'average'){

      # Take one more than requested as we are looking for a rate (difference between years)
      number_of_years_to_use_for_average <- min(future_averaging_period, nrow(available_inflation_data))

      years_to_extrapolate <- seq(max_year_available_without_extrapolation + 1, max_year_requested, 1)
      average_inflation <- available_inflation_data[1:number_of_years_to_use_for_average, "value"] %>% as.numeric %>% mean

      extrapolated_dataframe <- data.frame(value=rep(average_inflation, length(years_to_extrapolate)), years_to_extrapolate, stringsAsFactors = FALSE) %>% map_df(rev) %>% as.data.frame
      colnames(extrapolated_dataframe) <- c("value", "date")

      inflation_dataframe <- rbind(extrapolated_dataframe, available_inflation_data)

    } # End future / average

    if(extrapolate_future_method == 'rate'){

      years_to_extrapolate <- seq(max_year_available_without_extrapolation + 1, max_year_requested, 1)
      average_inflation <- future_rate

      extrapolated_dataframe <- data.frame(value=rep(average_inflation, length(years_to_extrapolate)), years_to_extrapolate, stringsAsFactors = FALSE) %>% map_df(rev) %>% as.data.frame
      colnames(extrapolated_dataframe) <- c("value", "date")

      inflation_dataframe <- rbind(extrapolated_dataframe, available_inflation_data)

    } # End future / rate

  } # End outermost else for extrapolating future




  # Extrapolating past
  min_year_requested <- min(to_date, from_date) # Note that this is still the 'to_date'; taking whatever values are provided into a date in the past
  min_year_available_without_extrapolation <- min(available_inflation_data$date)

  if(extrapolate_past & (min_year_requested >= min_year_available_without_extrapolation)) {
    warning(paste0("Past extrapolation not required as data available for ", country, " back to ", min_year_available_without_extrapolation))
    }


  if(extrapolate_past & (min_year_requested < min_year_available_without_extrapolation)) {

    if(extrapolate_past_method == 'average'){

      # Take one more than requested as we are looking for a rate (difference between years)
      number_of_years_to_use_for_average <- min(past_averaging_period, nrow(available_inflation_data))

      years_to_extrapolate <- seq(min_year_available_without_extrapolation - 1, min_year_requested, - 1) # Note these are in reverse
      average_inflation <- available_inflation_data[(nrow(available_inflation_data)-number_of_years_to_use_for_average + 1):nrow(available_inflation_data), "value"] %>% as.numeric %>% mean

      extrapolated_dataframe <- data.frame(value=rep(average_inflation, length(years_to_extrapolate)), years_to_extrapolate, stringsAsFactors = FALSE)
      colnames(extrapolated_dataframe) <- c("value", "date")

      inflation_dataframe <- rbind(available_inflation_data, extrapolated_dataframe)

    } # End past / average

    if(extrapolate_past_method == 'rate'){

      years_to_extrapolate <- seq(min_year_available_without_extrapolation - 1, min_year_requested, -1)
      average_inflation <- past_rate

      extrapolated_dataframe <- data.frame(value=rep(average_inflation, length(years_to_extrapolate)), years_to_extrapolate, stringsAsFactors = FALSE)
      colnames(extrapolated_dataframe) <- c("value", "date")

      inflation_dataframe <- rbind(available_inflation_data, extrapolated_dataframe)

    } # End past / rate

  } # End outermost else for extrapolating past




  # If dates are outside of avaiable data and no extrapolation is set
  if(max(to_date) > max(available_inflation_data$date) & !extrapolate_future) {
    stop(paste0("'to_date' (", to_date, ") is/contains a later date than the latest available data (",
                max(available_inflation_data$date), ").\nTry setting 'extrapolate_future' to TRUE or using an earlier 'to_date'"))
  }

  if(min(to_date) < min(available_inflation_data$date) & !extrapolate_past) {
    stop(paste0("'to_date' (", from_date, ") is/contains an earlier date than the earliest available data (",
                min(available_inflation_data$date), ").\nTry setting 'extrapolate_past' to TRUE or using a later 'to_date'"))
  }



  # This function takes a single from value and single to value and creates a multiplier
  # Note that we don't pass inflation_dataframe object to this as it confuses mapply/sapply and they
  # don't understand what to do with it
  # from_input <- from_date; to_input <- to_date
  make_multiplier <- function(from_input, to_input) {

    # Note diligent use of inequalities (rightly) prevent current year's inflation being applied
    # https://economics.stackexchange.com/questions/28972/convention-standard-methodology-for-calculating-real-prices
    # if(length(from_date) == 1) { from_input <- rep(from_input, length(price))}
    # if(length(to_date) == 1) { to_input <- rep(to_input, length(price))}

    inflation_dataframe %>%
      filter(date > from_input & date <= to_input | date < from_input & date >= to_input ) %>%
      .$value %>% {. / 100} %>% {. + 1} %>% { ifelse(from_input < to_input, prod(.), { 1 / prod(.) }) }
  }

  # from_input <- from_date
  # to_input <- to_date
  #
  # mapply(make_multiplier, from_input = from_date, to_input = to_date)

  multipliers <- mapply(make_multiplier, from_input = from_date, to_input = to_date)

  real_price <- price * multipliers

  real_price
}




#' @rdname adjust_for_inflation
#' @export
afi <- adjust_for_inflation

context("Outputs (or errors/warnings) are generated as expected")
# library(jsonlite)
# library(dplyr)
# library(lubridate)
# library(purrr)
# library(testthat)
# options(digits = 22)
# devtools::test()


#
# #----- URL conversion -----#
#
# test_that("url_all_results() correctly converts URL to JSON", {
#   original_url <- "http://api.worldbank.org/v2/country" # Note: no ?format=json on url
#   expect_gt(url_all_results(original_url) %>% fromJSON %>% .[[2]] %>% nrow, 50)
# })
#
#
#
# #----- Retieving all results (not just first page) from WB API -----#
#
# test_that("The number of total results is retreived (and not just 50 on first page)", {
#   original_url <- "http://api.worldbank.org/v2/country?format=json"
#   total_results <- original_url %>% fromJSON %>% .[[1]] %>% .$total
#   expect_gt(total_results, 50)
#
# })
#
# test_that("url_all_results() correctly retreives WB API data", {
#   original_url <- "http://api.worldbank.org/v2/country?format=json"
#   expect_gt(url_all_results(original_url) %>% fromJSON %>% .[[2]] %>% nrow, 50)
# })
#
#
#
# #----- Retieving countries from WB API -----#
#
# test_that("show_countries() retrieves countries as expected", {
#   expect_gt(show_countries() %>% nrow, 50)
# })
#
#
#
# #----- Identification of iso2Code/country_name -----#
#
# test_that("show_countries() retrieves countries as expected", {
#   expect_equal(country_input_type("AU", countries_dataframe), "iso2Code")
#   expect_equal(country_input_type("Australia", countries_dataframe), "country_name")
#   expect_equal(country_input_type("AustraTYPOlia", countries_dataframe), "invalid")
# })
#
#
#
# #----- Identification of iso2Code/country_name -----#
#
# test_that("iso2Code is identified as such and kept as iso2Code", {
#   country <- "AU"
#   country_input_type_string <- country_input_type(country, countries_dataframe)
#   convert_to_iso2Code(country_input_type_string, country) %>% expect_equal("AU")
# })
#
# test_that("country_name is identified as such and converted to iso2Code", {
#   country <- "Australia"
#   country_input_type_string <- country_input_type(country, countries_dataframe)
#   convert_to_iso2Code(country_input_type_string, country, countries_dataframe) %>% expect_equal("AU")
# })
#
# test_that("invalid country_name/iso2Code is identified as such", {
#   country <- "AustrTESTalia"
#   country_input_type_string <- country_input_type(country, countries_dataframe)
#   expect_error(convert_to_iso2Code(country_input_type_string, country),
#                paste0("'", country, "'", " is not a valid country input - select a valid country from show_countries()"))
# })
#
#
# #----- Testing retrieval of of inflation data using retrieve_inflation_data() -----#
#
# test_that("Inflation data is retrieved as expected for iso2Code input", {
#   country <- "AU"
#   expect_gt(retrieve_inflation_data(country) %>% .[[2]] %>% nrow, 50)
# })
#
# test_that("Inflation data is retrieved as expected for country_name input", {
#   country <- "Australia"
#   expect_gt(retrieve_inflation_data(country) %>% .[[2]] %>% nrow, 50)
# })
#
# test_that("Retrieval of inflation data for an invalid input fails with appropriate error message", {
#   country <- "AustraTESTlia"
#   expect_error(retrieve_inflation_data(country) %>% .[[2]] %>% nrow,
#                paste0("'", country, "'", " is not a valid country input - select a valid country from show_countries()"))
# })
#
#
#
# ###########################################
# ##### Testing adjust_for_inflation() ######
# ###########################################
#
#
#
#
# test_that("One price, one from date, one to date", {
#
#   price <- 10
#   from_date <- 2015
#
#   adjust_for_inflation(price, from_date, country, to_date = 2017,
#                        inflation_dataframe = inflation_dataframe,
#                        countries_dataframe = countries_dataframe) %>%
#     expect_equal(10.32505224053865866551)
#
# })
#
# # # Note: this doesn't will fail each time API data updates.
# # test_that("One price, one from date, one to date, extrapolating using 3 year average", {
# #
# #   price <- 10
# #   from_date <- 2022 # Some future date so not using real data
# #
# #   adjust_for_inflation(price, from_date, country, to_date = 2030,
# #                        inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
# #                        extrapolate_future_method = "average", future_averaging_period = 3) %>%
# #     expect_equal(11.33436825573669715084)
# #
# # })
# #
# # # Note: this doesn't will fail each time API data updates.
# # test_that("One price, one from date, one to date", {
# #
# #   price <- 10
# #   from_date <- 2025 # Some future date so not using real data
# #
# #   adjust_for_inflation(price, from_date, country, to_date = 2022,
# #                        inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
# #                        extrapolate_future_method = "average", future_averaging_period = 3) %>%
# #     expect_equal(9.541156191493925575742)
# #
# # })
#
#
#
#
#
# # NOT IN AUTOMATED TESTING
# # Omitting to_date (assumes current)
# # adjust_for_inflation(price, from_date, country,
# #                      inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
# #                      extrapolate_future_method = "average", future_averaging_period = 3)
#
#
#
#
#
# test_that("adjust_for_inflation() errors with a useful message when to_date is beyond available data", {
#
#   to_date <- 2050
#   from_date <- 2017
#
#   expect_error(
#     # Function
#     adjust_for_inflation(price, from_date, country, to_date = to_date,
#                          inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe)
#     ,
#     # Error message
#     paste0("'to_date' (", to_date, ") is/contains a later date than the latest available data (",
#            max(available_inflation_data$date), ").\nTry setting 'extrapolate_future' to TRUE or using an earlier 'to_date'"),
#
#     # Avoid regex: https://github.com/r-lib/testthat/issues/726
#     fixed = TRUE
#   )
# }
# )
#
#
#
#
# test_that("adjust_for_inflation() prompts user for number of years to use in extrapolating using average (future)", {
#
#   to_date <- 2030
#   extrapolate_future_method = "average"
#
#   expect_error(
#     # Function
#     adjust_for_inflation(price, from_date, country, to_date = to_date,
#                          inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
#                          extrapolate_future_method = "average")
#     ,
#     # Error message
#     paste0("Please specify how many years' average to use when extrapolating forward ('future_averaging_period'
#            can take any positive integer or 'all' to use an average of all available years' data"),
#
#     # Avoid regex: https://github.com/r-lib/testthat/issues/726
#     fixed = TRUE
#   )})
#
#
#
# # NOT IN AUTOMATED TESTING
# # adjust_for_inflation(price, from_date, country, to_date = 2030,
# #                      inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
# #                      extrapolate_future_method = "average", future_averaging_period = 3)
#
#
#
#
#
#
# test_that("Expect a future_rate if rate extrapolation is used (future)", {
#
#   to_date <- 2030
#   extrapolate_future_method = "average"
#   future_averaging_period <- 3
#
#
#   expect_error(
#     # Function
#     adjust_for_inflation(price, from_date, country, to_date = to_date,
#                          inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
#                          extrapolate_future_method = "rate", future_averaging_period = future_averaging_period)
#     ,
#     # Error message
#     paste0("Please specify the assumed rate of inflation for future periods (i.e. using 'future_rate' parameter)"),
#
#     # Avoid regex: https://github.com/r-lib/testthat/issues/726
#     fixed = TRUE
#   )
# }
# )
#
#
# # NOT IN AUTOMATED TESTING
# # adjust_for_inflation(price, from_date, country, to_date = 2030,
# #                      inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
# #                      extrapolate_future_method = "rate", future_rate = 3)
# # [1] 27.59076877199724009415
#
#
#
# #----- Testing past extrapolation -----#
#
# # Testing the past extrapolations (since past data shouldn't change and responses should be consistent)
#
#
#
#
# test_that("past extrapolation works when based on the last three available periods",
#           {
#             from_date <- 2017
#
#             adjust_for_inflation(price, from_date, country, to_date = 1930,
#                                 inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
#                                 extrapolate_past_method = "average", past_averaging_period = 3) %>%
#               expect_equal(0.3846295809055066405868)
#             }
#           )
#
#
# test_that("past extrapolation works with a rate",
#           {adjust_for_inflation(price, from_date = 1990, country, to_date = 1930,
#                                 inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
#                                 extrapolate_past_method = "rate", past_rate = 2.5) %>%
#               expect_equal(0.6568850636543038801207)
#           }
# )
#
#
#
# test_that("returns a useful message when past extrapolation is not required",
#           {expect_warning(adjust_for_inflation(price, from_date = "1991-04-14", country, to_date = 1961,
#                                 inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
#                                 extrapolate_past_method = "rate", past_rate = 2.5),
#               "Past extrapolation not required as data available for AU back to 1960")
#           }
# )
#
#
#
#
# test_that("extrapolation works with longer form date input",
#           {adjust_for_inflation(price, from_date = "1991-04-14", country, to_date = 1930,
#                                 inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
#                                 extrapolate_past_method = "rate", past_rate = 2.5) %>%
#               expect_equal(0.6120064931609506819754)
#           }
# )
#
#
# test_that("extrapolation works with longer form date input",
#           {adjust_for_inflation(price, from_date = "2009-04-09 13:09:39 AEST", country, to_date = 1930,
#                                 inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe,
#                                 extrapolate_past_method = "rate", past_rate = 2.5) %>%
#               expect_equal(0.3832127850909711752614)
#           }
# )
#
#
#
#
# ##################################
# ##### Testing vectorisation ######
# ##################################
#
#
#
# test_that("adjust_for_inflation() can handle a vector of price inputs",
#           {
#             price <- c(10, 11, 12, 15)
#             from_date <- 2007
#
#             adjust_for_inflation(price = price, from_date = from_date, to_date = 2017, country = country,
#                                 inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe) %>%
#               expect_equal(c(12.64429911856696620021, 13.90872903042366282023, 15.17315894228035944025, 18.96644867785045107667))
#           }
# )
#
#
#
#
#
# test_that("adjust_for_inflation() can handle a vector of price inputs and a vector of from dates",
#           {
#
#             price <- c(10, 10, 10, 10)
#             from_date <- c(2007, 2013, 2015, 2012)
#
#             adjust_for_inflation(price = price, from_date = from_date, to_date = 2017, country = country,
#                                 inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe) %>%
#               expect_equal(c(12.64429911856696620021, 10.74154589371981316503, 10.32505224053865866551, 11.00470180648354734387))
#           }
# )
#
#
#
#
#
#
#
#
# test_that("adjust_for_inflation() can handle a vector of price inputs, from dates and to dates",
#           {
#
#             price <- c(10, 10, 10, 10)
#             from_date <- c(2009, 2013, 2015, 2012)
#             to_date <- c(1997, 2008, 2007, 2017)
#
#             adjust_for_inflation(price = price, from_date = from_date, to_date = to_date, country = country,
#                                 inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe) %>%
#               expect_equal(c(7.269754768392368227126,  8.703291264538480120905,  8.100400659910437894951, 11.004701806483547343873))
#           }
# )
#
#
#
# test_that("vector inputs return same results as element inputs done separately",
#           {
#             price <- c(10, 10, 10, 10)
#             from_date <- c(2009, 2013, 2015, 2012)
#             to_date <- c(1997, 2008, 2007, 2017)
#
#             first <- adjust_for_inflation(price = price[1], from_date = from_date[1], to_date = to_date[1], country = country,
#                                           inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe)
#
#             second <- adjust_for_inflation(price = price[2], from_date = from_date[2], to_date = to_date[2], country = country,
#                                           inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe)
#
#             third <- adjust_for_inflation(price = price[3], from_date = from_date[3], to_date = to_date[3], country = country,
#                                           inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe)
#
#             fourth <- adjust_for_inflation(price = price[4], from_date = from_date[4], to_date = to_date[4], country = country,
#                                           inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe)
#
#
#             adjust_for_inflation(price = price, from_date = from_date, to_date = to_date, country = country,
#                                 inflation_dataframe = inflation_dataframe, countries_dataframe = countries_dataframe) %>%
#               expect_equal(c(first, second, third, fourth))
#           }
# )
#
#
#
# # devtools::test()
#

library(jsonlite)
library(dplyr)
library(purrr)
library(testthat)
# options(digits = 22)

# Note that this .R script doesn't know about functions in priceR
# So let's change that

# print(getwd())
# Note: this will be executed from: /tests/testthat"
# source("../../R/adjust_for_inflation.R")




# # Assign these variables once
# country <- "AU"
# inflation_dataframe <- retrieve_inflation_data(country)
# countries_dataframe <- show_countries()
#
#
# # Required for some error messages
# available_inflation_data <- inflation_dataframe %>% na.omit
#
# price <- 10
#

#' Round prices to the nearest specified increment
#'
#' @name round_to_nearest
#'
#' @usage round_to_nearest(amount, to_nearest)
#'
#' @param amount Price to be rounded
#' @param to_nearest Increment to which price is to be rounded to
#'
#'
#' @export
#'
#' @examples
#'
#' # Round to nearest 0.05 (5c)
#' library(dplyr)
#' prices <- c(4.45, 5.22, 0.16, 27.88, 112.19)
#' prices %>% round_to_nearest(0.05)
#'
#'
#' # Round to nearest $10
#' prices <- c(4.45, 5.22, 0.16, 27.88, 112.19)
#' prices %>% round_to_nearest(10)
#'
#'

round_to_nearest <- function(amount, to_nearest) {
  round(amount/to_nearest) * to_nearest
}



#' Round prices up to the nearest specified increment
#'
#' @name round_up_to_nearest
#'
#' @usage round_up_to_nearest(amount, to_nearest)
#'
#' @param amount Price to be rounded
#' @param to_nearest Increment to which price is to be rounded up to
#'
#'
#' @export
#'
#' @examples
#'
#' # Round up to nearest 0.05 (5c)
#' library(dplyr)
#' prices <- c(4.45, 5.22, 0.16, 27.88, 112.19)
#' prices %>% round_up_to_nearest(0.05)
#'
#'
#' # Round up to nearest $10
#' prices <- c(4.45, 5.22, 0.16, 27.88, 112.19)
#' prices %>% round_up_to_nearest(10)


round_up_to_nearest <- function(amount, to_nearest) {
  ceiling(amount/to_nearest) * to_nearest
}




#' Round prices down to the nearest specified increment
#'
#' @name round_down_to_nearest
#'
#' @usage round_down_to_nearest(amount, to_nearest)
#'
#' @param amount Price to be rounded
#' @param to_nearest Increment to which price is to be rounded down to
#'
#'
#' @export
#'
#' @examples
#'
#' # Round down to nearest 0.05 (5c)
#' library(dplyr)
#' prices <- c(4.45, 5.22, 0.16, 27.88, 112.19)
#' prices %>% round_down_to_nearest(0.05)
#'
#'
#' # Round down to nearest $10
#' prices <- c(4.45, 5.22, 0.16, 27.88, 112.19)
#' prices %>% round_down_to_nearest(10)


round_down_to_nearest <- function(amount, to_nearest) {
  floor(amount/to_nearest) * to_nearest
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currencies.R
\name{currencies}
\alias{currencies}
\title{Retrieve available currencies and their respective symbols/codes}
\usage{
currencies()
}
\value{
A data.frame of available currencies and their respective symbols/codes
}
\description{
Retrieve available currencies and their respective symbols/codes
}
\examples{
\dontrun{
# Display available currencies and their respective symbols/codes
currencies()
#                                description code
# 1              United Arab Emirates Dirham  AED
# 2                           Afghan Afghani  AFN
# 3                             Albanian Lek  ALL
# 4                            Armenian Dram  AMD
# 5            Netherlands Antillean Guilder  ANG
# 6                           Angolan Kwanza  AOA
# 7                           Argentine Peso  ARS
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjust_for_inflation.R
\name{adjust_for_inflation}
\alias{adjust_for_inflation}
\alias{afi}
\title{Convert nominal prices into real prices}
\usage{
adjust_for_inflation(price, from_date, country, to_date, inflation_dataframe,
countries_dataframe, extrapolate_future_method, future_averaging_period, future_rate,
extrapolate_past_method, past_averaging_period, past_rate)

afi(
  price,
  from_date,
  country,
  to_date,
  inflation_dataframe,
  countries_dataframe,
  extrapolate_future_method,
  future_averaging_period,
  future_rate,
  extrapolate_past_method,
  past_averaging_period,
  past_rate
)
}
\arguments{
\item{price}{A price (or prices).}

\item{from_date}{A date(s) from which the prices will be converted.}

\item{country}{A country or region in whose currency the prices are denominated.}

\item{to_date}{A date(s) to which the prices will be converted.}

\item{inflation_dataframe}{The R object (list) representing the JSON retrieved by calling retrieve_inflation_data().}

\item{countries_dataframe}{The R object (data.frame) representing the JSON retreived by calling show_countries().}

\item{extrapolate_future_method}{The extrapolation method that shall be used if extrapolation into the future is required. Options are 'average' or 'rate'.}

\item{future_averaging_period}{The number of recent periods to average in order to extrapolate forward (if 'average' is method being used).}

\item{future_rate}{An assumed rate of inflation to use for extrapolating forward (if 'rate' is method being used).}

\item{extrapolate_past_method}{The extrapolation method that shall be used if extrapolation from the earliest available data to some even earlier period is required.}

\item{past_averaging_period}{The number of periods back from the earliest available inflation data for a given country to average in order to extrapolate into the past (if 'average' is method being used).}

\item{past_rate}{An assumed rate of inflation to use for extrapolating from the earliest available data to some even earlier period (if 'rate' is method being used).}
}
\value{
A vector of inflation-adjusted prices
}
\description{
Inflate/deflate prices from any year to any year, using World Bank inflation data and assumptions only where necessary.
Typically used for converting past (nominal) values into current (real) values. This uses World Bank inflation data where available,
but allows for both historical and future assumptions in extrapolation.
}
\examples{
\dontrun{
# Assign these variables once
country <- "AU"
inflation_dataframe <- retrieve_inflation_data(country)
countries_dataframe <- show_countries()

# Convert $100 from 2005 into 2017 dollars

adjust_for_inflation(100, 2005, country, to_date = 2017,
inflation_dataframe = inflation_dataframe,
countries_dataframe = countries_dataframe)

# [1] 133.9861 # i.e. $100 in 2005 had the same purchasing power as $133.99 in 2017
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currencies.R
\name{retrieve_historical_rates}
\alias{retrieve_historical_rates}
\title{Retrieve historical exchange rates}
\usage{
retrieve_historical_rates(from, to, start_date, end_date)
}
\arguments{
\item{from}{A currency code (see currencies() for supported codes)}

\item{to}{A currency code}

\item{start_date}{A start date (of the form "2010-01-01")}

\item{end_date}{An end date}
}
\value{
A data.frame containing exchange rate data for select currency pair
}
\description{
Retrieves historical exchange rates between a currency pair - retrieves max. 365 days' data
}
\examples{
\dontrun{
# Note date range >365 days', yet only 365 days' returned.
# Use historical_exchange_rates() for > 365 days'.
 priceR:::retrieve_historical_rates("USD", to = "AUD",
                                    start_date = "2018-01-01",
                                    end_date = "2020-06-30")
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.R
\name{format_dollars}
\alias{format_dollars}
\title{Make numeric currency values human readable}
\usage{
format_dollars(amount, digits)
}
\arguments{
\item{amount}{Price or vector of prices (character, numeric, or integer)}

\item{digits}{The number of decimal places. Set equal to 2 to include cents (defaults to 0 i.e. whole dollars)}
}
\value{
A character vector
}
\description{
Make numeric currency values human readable
}
\examples{

# format_dollars("2423562534234")
# "$2,423,562,534,234"

# format_dollars("2423562534234.876", 0)
# "$2,423,562,534,234"

# format_dollars("2423562534234.876", 2)
# "$2,423,562,534,234.88"

# format_dollars("2423562534234", 2)
# "$2,423,562,534,234.00"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currencies.R
\name{make_dates}
\alias{make_dates}
\title{Creates date ranges so as to batch up large API calls into many smaller ones}
\usage{
make_dates(start_date, end_date, n_days)
}
\arguments{
\item{start_date}{A start date (of the form "2010-01-01")}

\item{end_date}{An end date}

\item{n_days}{The maximum number of days in each period}
}
\value{
A data.frame containing start and end dates for periods of length no longer than n_days
}
\description{
Creates date ranges so as to batch up large API calls into many smaller ones
}
\examples{

# Simple test
start_date = "2010-01-01"
end_date = "2020-06-30"
n_days = 365
priceR:::make_dates(start_date, end_date, n_days)

# With lots of periods
start_date = "2010-01-01"
end_date = "2020-06-30"
n_days = 20
priceR:::make_dates(start_date, end_date, n_days)

# Less than one period
start_date = "2020-01-01"
end_date = "2020-06-30"
n_days = 365
priceR:::make_dates(start_date, end_date, n_days)

# 366 days (note 2020 was a leap year)
start_date = "2019-07-30"
end_date = "2020-07-29"
n_days = 365
priceR:::make_dates(start_date, end_date, n_days)

# 365 days
start_date = "2019-07-30"
end_date = "2020-07-28"
n_days = 365
priceR:::make_dates(start_date, end_date, n_days)

# 1095 days (3 years)
start_date = "2019-07-30"
end_date = "2022-07-28"
n_days = 365
priceR:::make_dates(start_date, end_date, n_days)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currencies.R
\docType{data}
\name{currency_info}
\alias{currency_info}
\title{Information for each of 191 currencies}
\format{
An object of class \code{data.frame} with 191 rows and 15 columns.
}
\usage{
currency_info
}
\value{
A data.frame containing currency information for 191 currencies.
    Currency information includes: name, iso code, currency symbol (and
    alternative symbols if applicable), subunit, number of subunits per major
    unit, whether the currency symbol ought to appear before or after the
    number of units, display format, html entity, decimal mark,
    thousands separator, iso numeric, and smallest denomination.
}
\description{
Information for each of 191 currencies
}
\examples{
head(currency_info)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjust_for_inflation.R
\name{country_input_type}
\alias{country_input_type}
\title{Determines whether country input is a country name or iso2Code}
\usage{
country_input_type(country_input, countries_dataframe)
}
\arguments{
\item{country_input}{A country/region the user wishes to validate (string) E.g. "Australia".}

\item{countries_dataframe}{A dataframe containing available iso2Code and country_name
(see show_countries()).}
}
\value{
A character vector
}
\description{
Determines whether a string is a country name, an iso2Code, or invalid (not a World Bank API
country/region)
}
\examples{

\dontrun{
# Assign so as to save on API calls - recommended
countries_dataframe <- show_countries()

  country <- "Australia"
  country_input_type(country, countries_dataframe)
# [1] "country_name"

  country <- "AU"
  country_input_type(country, countries_dataframe)
# [1] "iso2Code"

  country <- "something other than a valid country name or iso2Code"
  country_input_type(country, countries_dataframe)
# [1] "invalid"
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currencies.R
\name{pminmax}
\alias{pminmax}
\title{Removes redundant API calls of currency pairs. That is, revmoes the need
to for separate calls for both
`from = EUR, to = USD` and `from = USD, to = EUR`}
\usage{
pminmax(x, y)
}
\arguments{
\item{x}{A currency code (see currencies() for supported codes)}

\item{y}{A currency code}
}
\value{
A character vector
}
\description{
Removes redundant API calls of currency pairs. That is, revmoes the need
to for separate calls for both
`from = EUR, to = USD` and `from = USD, to = EUR`
}
\examples{

# See: https://stackoverflow.com/a/73254014/9059865



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjust_for_inflation.R
\name{show_countries}
\alias{show_countries}
\title{Show available country codes}
\usage{
show_countries()
}
\value{
A data.frame of countries available to query using the World Bank API
}
\description{
`show_countries` calls the World Bank API and retrieves a list of available countries and regions
}
\examples{

# Simply call show_countries() to receive a dataframe of all countries (and regions) and their
# iso2Code

# show_countries()
#       iso2Code         country_name
#   1         AW                Aruba
#   2         AF          Afghanistan
#   3         A9               Africa
#   4         AO               Angola
#   Etc


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currencies.R
\name{from_to_dates_rates}
\alias{from_to_dates_rates}
\title{Wrapper around `priceR::historical_exchange_rates()` with slight modifications
to structure of inputs and output}
\usage{
from_to_dates_rates(from, to, dates)
}
\arguments{
\item{from}{A currency code (see currencies() for supported codes)}

\item{to}{A currency code}

\item{dates}{A list of date ranges}
}
\value{
A data.frame with two columns: date (of class Date), and rate
    (of class numeric).
}
\description{
Wrapper around `priceR::historical_exchange_rates()` with slight modifications
to structure of inputs and output
}
\examples{
\dontrun{
library(lubridate)
from_to_dates_rates("AUD", "USD", dates = list(today()-10, today()))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currencies.R
\name{exchange_rate_latest}
\alias{exchange_rate_latest}
\title{Retrieve the latest exchange rates between the provided currency code}
\usage{
exchange_rate_latest(currency)
}
\arguments{
\item{currency}{A currency code (see currencies() for supported codes)}
}
\value{
A data.frame containing the latest exchange rates between the provided currency code
    and each other available currency
}
\description{
Retrieve the latest exchange rates between the provided currency code
}
\examples{
\dontrun{

exchange_rate_latest("AUD")
# Daily AUD exchange rate as at end of day 2020-07-27 GMT
#     currency one_aud_is_equivalent_to
# 1        AED                  2.61894
# 2        AFN                 54.47724
# 3        ALL                 75.51799
# 4        AMD                343.40193
# 5        ANG                  1.26829
# 6        AOA                400.54604


# Defaults to USD
exchange_rate_latest()
#     currency one_USD_is_equivalent_to
# 1        AED                   3.6730
# 2        AFN                  76.4035
# 3        ALL                 105.9129
# 4        AMD                 481.6162
# 5        ANG                   1.7788
# 6        AOA                 561.7599

# It can also accept other base rates
exchange_rate_latest("AUD")
#    currency one_AUD_is_equivalent_to
# 1       AED                  2.31619
# 2       AFN                 48.69229
# 3       ALL                 63.87806
# 4       AMD                260.72150
# 5       ANG                  1.13675
# 6       AOA                522.76772
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.R
\name{currency_to_numeric}
\alias{currency_to_numeric}
\title{Convert human readable currencies into numeric data}
\usage{
currency_to_numeric(currency_text)
}
\arguments{
\item{currency_text}{Price or vector of prices}
}
\value{
A numeric vector
}
\description{
Convert human readable currencies into numeric data
}
\examples{
library(dplyr)
c("$134,345.05", "£22", "¥30000") \%>\% currency_to_numeric()
# [1] 134345     22  30000
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjust_for_inflation.R
\name{convert_to_iso2Code}
\alias{convert_to_iso2Code}
\title{Convert any country input into its iso2Code}
\usage{
convert_to_iso2Code(country_input_type_string, country, countries_dataframe)
}
\arguments{
\item{country_input_type_string}{Either "country_name" or "iso2Code" - use country_input_type(country, countries_dataframe) to determine or assign manually.}

\item{country}{A country/region name or iso2Code.}

\item{countries_dataframe}{The output of show_countries()}
}
\value{
A character vector containing a valid iso2Code
}
\description{
`convert_to_iso2Code` accepts the type of country input and the country, and returns the relevant iso2Code
}
\examples{

\dontrun{

# Assign so as to save on API calls (recommended)
countries_dataframe <- show_countries()
country <- "Australia"
country_input_type_string <- country_input_type(country, countries_dataframe)
convert_to_iso2Code(country_input_type_string, country, countries_dataframe)
# [1] "AU"

country <- "AU"
country_input_type_string <- country_input_type(country, countries_dataframe)
convert_to_iso2Code(country_input_type_string, country, countries_dataframe)
# [1] "AU"


}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjust_for_inflation.R
\name{retrieve_inflation_data}
\alias{retrieve_inflation_data}
\title{Retrieve historical inflation data}
\usage{
retrieve_inflation_data(country, countries_dataframe)
}
\arguments{
\item{country}{A country_name or iso2code (see show_countries() for complete list of available inputs).}

\item{countries_dataframe}{The output from show_countries(). It is optional, but if not provided, it will be retrieved via the API.}
}
\value{
A data.frame containing inflation data from World Bank API for specified country
}
\description{
Retrieve inflation data for any country/region (using iso2Code or country_name)
}
\examples{
\dontrun{
# Retrieve inflation data for any country (or iso2Code)
country <- "AU"
inflation_dataframe <- retrieve_inflation_data(country)

country <- "Australia"
countries_dataframe <- show_countries()
inflation_dataframe <- retrieve_inflation_data(country, countries_dataframe)
}
# inflation_dataframe
#    indicator.id                       indicator.value country.id country.value     value
#  FP.CPI.TOTL.ZG Inflation, consumer prices (annual \%)         AU     Australia      <NA>
#  FP.CPI.TOTL.ZG Inflation, consumer prices (annual \%)         AU     Australia   1.94864
#  FP.CPI.TOTL.ZG Inflation, consumer prices (annual \%)         AU     Australia   1.27699
#  FP.CPI.TOTL.ZG Inflation, consumer prices (annual \%)         AU     Australia   1.50836
#  Etc


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currencies.R
\name{historical_exchange_rates}
\alias{historical_exchange_rates}
\title{Retrieve historical exchange rates}
\usage{
historical_exchange_rates(from, to, start_date, end_date)
}
\arguments{
\item{from}{A currency code (see currencies() for supported codes)}

\item{to}{A currency code}

\item{start_date}{A start date (of the form "2010-01-01")}

\item{end_date}{An end date}
}
\value{
A data.frame containing exchange rate data for select currency pair
}
\description{
Retrieves historical exchange rates between a currency pair
}
\examples{
\dontrun{
Retrieve AUD to USD exchange rates
au <- historical_exchange_rates(from = "AUD", to = "USD",
                          start_date = "2010-01-01", end_date = "2020-06-30")

# Retrieve AUD to EUR exchange rates
ae <- historical_exchange_rates(from = "AUD", to = "EUR",
                          start_date = "2010-01-01", end_date = "2020-06-30")

# Combine
cur <- au \%>\% left_join(ae, by = "date")

head(cur)
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/round_to_nearest.R
\name{round_up_to_nearest}
\alias{round_up_to_nearest}
\title{Round prices up to the nearest specified increment}
\usage{
round_up_to_nearest(amount, to_nearest)
}
\arguments{
\item{amount}{Price to be rounded}

\item{to_nearest}{Increment to which price is to be rounded up to}
}
\description{
Round prices up to the nearest specified increment
}
\examples{

# Round up to nearest 0.05 (5c)
library(dplyr)
prices <- c(4.45, 5.22, 0.16, 27.88, 112.19)
prices \%>\% round_up_to_nearest(0.05)


# Round up to nearest $10
prices <- c(4.45, 5.22, 0.16, 27.88, 112.19)
prices \%>\% round_up_to_nearest(10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currencies.R
\name{convert_currencies}
\alias{convert_currencies}
\title{Convert Currencies}
\usage{
convert_currencies(
  price_start,
  from,
  to,
  date = lubridate::today(),
  floor_unit = "day"
)
}
\arguments{
\item{price_start}{Numeric vector of prices in terms of `from` currenc(ies).}

\item{from}{Character vector of currenc(ies) of `price_start`.}

\item{to}{Character vector of currenc(ies) to convert `price_start` to.}

\item{date}{Date vector specifying date of exchange rate to use.}

\item{floor_unit}{Character string. Default is "day" meaning that `date` will
be converted based on daily conversion rates. Changing to "week" will
change conversions to be based on the start of the week of `date`.}
}
\value{
Numeric vector of `price_start` now in the `to` currenc(ies).
}
\description{
Vectorized approach to converting prices across potentially different dates
and between different currencies.
}
\examples{
\dontrun{
library(dplyr)

sales_transactions <- tibble(
  local_price = c(100, 80, 9200, 90),
  local_currency = c("USD", "EUR", "JPY", "USD"),
  final_currency = c("EUR", "USD", "USD", "JPY"),
  date_transaction = lubridate::ymd(c(20200601, 20200609, 20200614, 20200623))
)
# Some made-up sales transactions of different values and currencies
sales_transactions \%>\%
  mutate(
    converted_price = convert_currencies(
      price_start = local_price,
      from = local_currency,
      to = final_currency,
      date = date_transaction
    )
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_salary.R
\name{extract_salary}
\alias{extract_salary}
\title{Extract numeric salary from text data}
\usage{
extract_salary(salary_text, exclude_below, exclude_above, salary_range_handling,
include_periodicity, hours_per_workday, days_per_workweek, working_weeks_per_year)
}
\arguments{
\item{salary_text}{A character string, or vector of character strings.}

\item{exclude_below}{A lower bound. Anything lower than this number will be replaced with NA.}

\item{exclude_above}{An upper bound. Anything above this number will be replaced with NA.}

\item{salary_range_handling}{A method of handling salary ranges. Defaults to returning an average of the range; can also be set to "max" or "min".}

\item{include_periodicity}{Set to TRUE to return an additional column stating the detected peridicity in the character string. Periodicity is assumed to be 'Annual' unless evidence is found to the contrary.}

\item{hours_per_workday}{Set assumed number of hours in the workday. Only affects annualisation of rates indentified as Daily. Default is 8 hours.}

\item{days_per_workweek}{Set assumed number of days per workweek. Only affects annualisation of rates indentified as Daily. Default is 5 days.}

\item{working_weeks_per_year}{Set assumed number of working weeks in the year. Only affects annualisation of rates indentified as Daily or Weekly. Default is 50 weeks.}
}
\value{
A data.frame of 1 column, or 2 columns if include_periodicity is set to TRUE
}
\description{
Extract numeric salary from text data. `extract_salary` automatically converts weekly and hourly rates to amounts per annum.
}
\examples{

# Provide a salary string and 'extract_salary' and will extract the salary and return it
extract_salary("$160,000 per annum")
# 160000


# If a range is present, the average will be taken by default
extract_salary("$160,000 - $180000.00 per annum")
# 170000


# Take the 'min' or 'max' of a salary range by setting salary_range_handling parameter accordingly
extract_salary("$160,000 - $180000.00 per annum", salary_range_handling = "min")
# 160000


# Extract salaries from character string(s)
annual_salaries <- c("$160,000 - $180000.00 per annum",
                     "$160000.00 - $180000.00 per annum",
                     "$145000 - $155000.00 per annum",
                     "$70000.00 - $90000 per annum",
                     "$70000.00 - $90000.00 per annum plus 15.4\% super",
                     "$80000.00 per annum plus 15.4\% super",
                     "60,000 - 80,000",
                     "$78,686 to $89,463 pa, plus 15.4\% superannuation",
                     "80k - 100k")

extract_salary(annual_salaries)
# 170000 170000 150000  80000  53338  40008  70000  56055  90000
# Note the fifth, sixth, and eighth elements are averages including '15' (undesirable)
# Using exclude_below parameter avoids this (see below)

# Automatically detect, extract, and annualise daily rates
daily_rates <- c("$200 daily", "$400 - $600 per day", "Day rate negotiable dependent on experience")
extract_salary(daily_rates)
# 48000 120000     NA


# Automatically detect, extract, and annualise hourly rates
hourly_rates <- c("$80 - $100+ per hour", "APS6/EL1 hourly rate contract")
extract_salary(hourly_rates)
# 172800   6720
# Note 6720 is undesirable. Setting the exclude_below and exclude_above sensibly avoids this


salaries <- c(annual_salaries, daily_rates, hourly_rates)


# Setting lower and upper bounds provides a catch-all to remove unrealistic results
# Out of bounds values will be converted to NA
extract_salary(salaries, exclude_below = 20000, exclude_above = 600000)
# 170000 170000 150000  80000  80000  80000  70000  84074  90000  48000 120000     NA 172800     NA


# extract_salary automatically annualises hourly and daily rates
# It does so by making assumptions about the number of working weeks in a year,
# days per workweek, and hours per workday
# And the assumed number of hours per workday can be changed from the default (8)
# The assumed number of workdays per workweek can be changed from the default (5)
# The assumed number of working weeks in year can be changed from the default (50)
# E.g.
extract_salary(salaries, hours_per_workday = 7, days_per_workweek = 4,
               working_weeks_per_year = 46, exclude_below = 20000)
# 170000 170000 150000  80000  53338  40008  70000  56055  90000  36800  92000     NA 115920     NA


# To see which salaries were detected as hourly or weekly, set include_periodicity to TRUE
extract_salary(salaries, include_periodicity = TRUE, exclude_below = 20000)

# salary periodicity
# 1  170000      Annual
# 2  170000      Annual
# 3  150000      Annual
# 4   80000      Annual
# 5   80000      Annual
# 6   80000      Annual
# 7   70000      Annual
# 8   84074      Annual
# 9   90000      Annual
# 10  48000       Daily
# 11 120000       Daily
# 12     NA       Daily
# 13 172800      Hourly
# 14     NA      Hourly


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.R
\name{format_currency}
\alias{format_currency}
\title{Make numeric currency values human readable}
\usage{
format_currency(amount, symbol, digits)
}
\arguments{
\item{amount}{Price or vector of prices (character, numeric, or integer)}

\item{symbol}{Symbol to prepend to amount (e.g. $) see: currency_characters()}

\item{digits}{The number of decimal places. Set equal to 2 to include cents (defaults to 0 i.e. whole major currency units)}
}
\value{
A character vector
}
\description{
Make numeric currency values human readable
}
\examples{

# format_currency("2423562534234", "$")
# "$2,423,562,534,234"

# format_currency("2423562534234.876", "$", 0)
# "$2,423,562,534,234"

# format_currency("2423562534234.876", "$", 2)
# "$2,423,562,534,234.88"

# format_currency("2423562534234", "¥", 2)
# "¥2,423,562,534,234.00"

# format_currency() is vectorized and can accept vector arguments
format_currency(c("2423562534234", "20"), c("¥", "$"), c(1, 2))
# "¥2,423,562,534,234.0" "$20.00"

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjust_for_inflation.R
\name{url_all_results}
\alias{url_all_results}
\title{Generate a World Bank API URL that will return all results for a given indicator in JSON format}
\usage{
url_all_results(original_url)
}
\arguments{
\item{original_url}{A World Bank API URL. E.g. "https://api.worldbank.org/v2/country".}
}
\value{
A character vector
}
\description{
results and returns JSON format
}
\examples{

# Provide a World Bank API URL and `url_all_results` will convert it into one with all results
# for that indicator
\dontrun{
  original_url <- "https://api.worldbank.org/v2/country" # Note: no ?format=json on url
  url_all_results(original_url)
}
# "https://api.worldbank.org/v2/country?format=json&per_page=304"



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.R
\name{currency_characters}
\alias{currency_characters}
\title{Provide currency characters}
\usage{
currency_characters()
}
\value{
A character vector of currency symbols
}
\description{
Provide currency characters
}
\examples{

currency_characters()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currencies.R
\name{append_exchangeratehost_access_key}
\alias{append_exchangeratehost_access_key}
\title{Retrieves exchangerate.host (forex) API key from R environment variables
and appends to API call}
\usage{
append_exchangeratehost_access_key(url)
}
\arguments{
\item{url}{A URL representing an API endpoint}
}
\value{
The input URL with API access key appended as a URL parameter
}
\description{
Retrieves exchangerate.host (forex) API key from R environment variables
and appends to API call
}
\examples{
\dontrun{
base_Url <- "https://api.exchangerate.host/latest?base=USD"
base_Url \%>\% append_exchangeratehost_access_key

# [1] "https://api.exchangerate.host/latest?base=USD&access_key=7e5e3140140bd8e4f4650cc41fc772c0"

}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/currencies.R
\name{display_api_info}
\alias{display_api_info}
\title{Display link to further information}
\usage{
display_api_info()
}
\description{
Display link to further information
}
\examples{
\dontrun{
# Display a message indicating where further documentation can be found
}

}
